/*
 * The Helium Toolkit
 *
 * Copyright (C) 1996-2000 Edouard Thiel <Edouard.Thiel@lif.univ-mrs.fr>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License; see http://www.lif-sud.univ-mrs.fr/~thiel/helium/LICENSE
*/

/*
 * gui/gltb.h - 10/10/2000
 *
 * 
*/


#ifndef HELIUM__GLTB_H
#define HELIUM__GLTB_H

                        
typedef struct {
        GLdouble	beginx, beginy,		/* Position de la souris */
        		fovy_val, 		/* Angle de prise de vue */
        		fovy_min, fovy_max,
                 	zdist_val, 		/* Distance  l'observateur */
                        zdist_min, zdist_max,
                 	quat[4];		/* Quaternion pour rotations */
} He_trackball;


/* PUBLIC */
extern void HeTbInitRotations (He_trackball *tb);
extern void HeTbInitFovy (He_trackball *tb, GLdouble val, GLdouble min, GLdouble max);
extern void HeTbInitZdist (He_trackball *tb, GLdouble val, GLdouble min, GLdouble max);
extern void HeTbMemoPointer (He_trackball *tb, He_event *hev);
extern GLdouble HeTbGetFovy (He_trackball *tb);
extern GLdouble HeTbGetZdist (He_trackball *tb);
extern void HeTbMultRotations (He_trackball *tb);
extern void HeTbEventRotations (He_trackball *tb, He_node *hn, He_event *hev);
extern void HeTbEventFovy (He_trackball *tb, He_node *hn, He_event *hev);
extern void HeTbEventZdist (He_trackball *tb, He_node *hn, He_event *hev);


/* PRIVE */
extern void HeTb_vzero (GLdouble *v);
extern void HeTb_vset (GLdouble *v, GLdouble x, GLdouble y, GLdouble z);
extern void HeTb_vsub (GLdouble *src1, GLdouble *src2, GLdouble *dst);
extern void HeTb_vcopy (GLdouble *v1, GLdouble *v2);
extern void HeTb_vcross (GLdouble *v1, GLdouble *v2, GLdouble *cross);
extern GLdouble HeTb_vlength (GLdouble *v);
extern void HeTb_vscale (GLdouble *v, GLdouble div);
extern void HeTb_vnormal (GLdouble *v);
extern GLdouble HeTb_vdot (GLdouble *v1, GLdouble *v2);
extern void HeTb_vadd (GLdouble *src1, GLdouble *src2, GLdouble *dst);
extern void HeTb_trackball (GLdouble q[4], GLdouble p1x, GLdouble p1y, 
                                           GLdouble p2x, GLdouble p2y);
extern void HeTb_axis_to_quat (GLdouble a[3], GLdouble phi, GLdouble q[4]);
extern GLdouble HeTb_tb_project_to_sphere (GLdouble r, GLdouble x, GLdouble y);
extern void HeTb_add_quats (GLdouble q1[4], GLdouble q2[4], GLdouble dest[4]);
extern void HeTb_normalize_quat (GLdouble q[4]);
extern void HeTb_build_rotmatrix (GLdouble m[4][4], GLdouble q[4]);


#endif /* HELIUM__GLTB_H */

