/*
 * The Helium Toolkit
 *
 * Copyright (C) 1996-2000 Edouard Thiel <Edouard.Thiel@lif.univ-mrs.fr>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License; see http://www.lif-sud.univ-mrs.fr/~thiel/helium/LICENSE
*/

/*
 * glx/glutils.c - 16/09/1999
 *
 * Divers GL
*/

#include <heliumGL.h>


/*------------------- I N T E R F A C E - P U B L I Q U E --------------------*/

/*
 * Demande au serveur X si l'extension GLX est disponible.
 * Appele' automatiquement a la premiere creation d'un GLArea.
 *
 * Renvoie 0 succes, -1 erreur.
*/

int HeQueryGL ()
{
    return glXQueryExtension (he_display, NULL, NULL) == True ? 0 : -1;
}


/*
 * Cree un contexte GL.
 * Appele' automatiquement par HeCreateGLArea().
*/
void HeNewGLXContext (
	int *gl_attr_list, GLXContext share_context, int direct_draw, 
        GLXContext *glxcontext_return, Visual **visual_return, 
        int *depth_return, int *doublebuffer_return)
{
    XVisualInfo *vi;
    
    *depth_return = he_default_depth;
    *visual_return = he_default_visual;
    *glxcontext_return = NULL;

    vi = glXChooseVisual (he_display, he_screen, gl_attr_list);
    
    if (vi == NULL){
	HeError ("HeNewGLContext: can't choose visual for gl_attr_list\n");
	return;
    }
    
    *glxcontext_return = glXCreateContext (he_display, vi, 
    	share_context, direct_draw ? True : False);
  
    *visual_return = vi->visual;
    *depth_return = vi->depth;
    
    /* Dtermination double buffer */
    if (glXGetConfig (he_display, vi,
        GLX_DOUBLEBUFFER, doublebuffer_return) != 0)
    {
        *doublebuffer_return = FALSE;
	HeError ("HeNewGLContext: can't check double buffer\n");
    }
    
    XFree(vi);
}

