/*
 * The Helium Toolkit
 *
 * Copyright (C) 1996-2000 Edouard Thiel <Edouard.Thiel@lif.univ-mrs.fr>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License; see http://www.lif-sud.univ-mrs.fr/~thiel/helium/LICENSE
*/

/*
 * gui/atomes.c - 10/09/1999
 *
 * 
*/

#include <helium.h>

Atom he_atom_mess, he_atom_sig, he_atom_wmdeco, he_atom_protoc,
     he_atom_delwin, he_atom_leader, he_atom_targets, he_atom_sel0,
     he_atom_clipboard;


/*------------------- I N T E R F A C E - P U B L I Q U E --------------------*/

/*
 * Attache une valeur de 32 bits a un Window.
*/

void HeAtomSet32 (Window win, Atom atom, Atom type, int val)
{
    XChangeProperty (he_display, win, atom, type,
	32, PropModeReplace, (Uchar *) &val, 1);
}


/*
 * Retrouve une valeur de 32 bits attache'e a un Window.
 * Renvoie -1 si erreur, l'entier val si succes.
 *
 * Silencieux.
*/

int HeAtomGet32 (Window win, Atom atom, Atom type)
{
    Atom type_return;
    int format_return;
    Ulong nitems;
    Ulong bytes_after;
    int val, err;
    unsigned char *prop_return;

    /* ATTENTION, cette fonction fait un malloc sur prop_return.
     * On doit toujours faire un XFree apres, sauf en cas d'erreur.
    */
    err = XGetWindowProperty (he_display, win, atom,
	0L, 1L, False, type,
	&type_return, &format_return, &nitems, &bytes_after,
	&prop_return);

    if (err != Success || nitems == 0 || prop_return == 0) 
	return -1;

    /* RQ si on n'avait pas teste' ces erreurs avant, 
       cette ligne pourrait provoquer un SIGSEGV */
    val = *prop_return;
    
    /* On libere la memoire */
    XFree (prop_return);

    /* Test si on a pu trouver l'atome avec le type demande' */
    if (type_return != type)
	return -1;

    return val;
}


/*
 * Retrouve une propriete attache'e a un Window.
 * Renvoie 0 succes, -1 si erreur.
 *
 * ATTENTION, cette fonction fait un malloc sur data.
 * On doit toujours faire un XFree(data) apres l'appel, sauf en cas d'erreur.
 *
 * Silencieux.
*/

int HeAtomGetProp (Window win, Atom prop, int free_prop,
	Atom *type, int *format, Ulong *nitems, Uchar **data)
{
    int err;
    long taille32;
    Ulong bytes_after;

    /* Appel avec None pour connaitre la taille de la propriete */
    err = XGetWindowProperty (he_display, win, prop,
	0L, 0L, False, None,
	type, format, nitems, &bytes_after, data);

    if (err != Success || type == None) return -1;

    taille32 = bytes_after / 4 + 1;

    /* Appel avec AnyPropertyType pour recuperer la propriete */
    err = XGetWindowProperty (he_display, win, prop,
	0L, taille32, free_prop, AnyPropertyType,
	type, format, nitems, &bytes_after, data);

    if (err != Success || type == None) return -1;
    
    /* On n'a pas recupere la propriete complete */
    if (bytes_after != 0) { XFree (data); return -1; }
    
    return 0;
}


/*
 * Affiche le nom d'un atome
*/

void HePrintAtomName (Atom atom)
{
    char *s;
    s = XGetAtomName (he_display, atom); 
    printf ("%s", s); XFree (s);
}


/*-------------------- F O N C T I O N S - P R I V E E S ---------------------*/

/*
 * Creation des atomes.
*/

void HeAtomInit ()
{
    /* Atome pour un message client envoye' par HeSendClientMessage() */
    he_atom_mess = XInternAtom (he_display, "HELIUM_MESS", False);

    /* Atome pour un message signal */
    he_atom_sig = XInternAtom (he_display, "HELIUM_SIG", False);

    /* Atome pour stocker dans he_root_window la taille de la deco */
    he_atom_wmdeco = XInternAtom (he_display, "HELIUM_WMDECO", False);

    /* Atome pour protocol avec le WM */
    he_atom_protoc = XInternAtom (he_display, "WM_PROTOCOLS", False);

    /* Atome protegeant un Window du Quit */
    he_atom_delwin = XInternAtom (he_display, "WM_DELETE_WINDOW", False);

    /* Atome pour le Window Group */
    he_atom_leader = XInternAtom (he_display, "WM_CLIENT_LEADER", False);

    /* Atomes pour la Selection */
    he_atom_targets = XInternAtom (he_display, "TARGETS", False);
    he_atom_sel0 = XInternAtom (he_display, "_XT_SELECTION_0", False);
    he_atom_clipboard = XInternAtom (he_display, "CLIPBOARD", False);
}


