/*
 * The Helium Toolkit
 *
 * Copyright (C) 1996-2000 Edouard Thiel <Edouard.Thiel@lif.univ-mrs.fr>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License; see http://www.lif-sud.univ-mrs.fr/~thiel/helium/LICENSE
*/

/*
 * gui/canvas.c - 28/06/1999
 *
 * 
*/

#include <helium.h>

/* Type et union du package 
*/
#define  OBJ_TYPE	Obj_canvas
#define  OBJ_UNION	obj.canvas

static He_method he_meth_canvas;


/*------------------- I N T E R F A C E - P U B L I Q U E --------------------*/

/*
 *  Cration d'un canvas.
 *  Renvoie le noeud ou NULL.
*/

He_node *HeCreateCanvas (He_node *owner)
{
    He_node *hn;
    OBJ_TYPE *ob;

    HeCanvasAutoInit();

    /* On cree le noeud */
    hn = HeBaseCreate (owner, &he_meth_canvas);
    if (hn == NULL) return NULL;

    /* On cree l'objet */
    ob = hn->OBJ_UNION = malloc (sizeof (OBJ_TYPE));
    if (ob == NULL) {
	HeError ("HeCreateCanvas: malloc error\n");
	return NULL;
    }

    /* Init des champs du noeud */
    hn->win_border = 1;
    
    /* init des champs de l'objet */
    ob->event_proc = NULL;
    ob->resize_proc = NULL;
    ob->repaint_proc = NULL;
    HeTipInitInfo (&ob->tip_info);
    
    ob->old_xm = -1;    /* pour detecter les resize ; ici on */
    ob->old_ym = -1;    /* force un resize  la creation.    */

    ob->dbuf = 0;       /* pas de double buffer */
    ob->back_buf = None;

    /* Creation du Window avec Xlib */
    hn->win = XCreateSimpleWindow (he_display, 
	hn->owner->win,
	hn->xb, hn->yb, hn->xm, hn->ym, 
	hn->win_border,
	he_black,
	he_white);

    /* Proprit du window */
    XSetStandardProperties (he_display, hn->win, "", "",
	0, he_argv, he_argc, NULL);
    HeEventMask (hn->win);
    HeWinListInsert (hn);
        
    /* (a l'init generale)
         Cursor cursor_pencil = XCreateFontCursor (he_display, XC_pencil);
      XDefineCursor (he_display, hn->win, cursor_pencil); */
    
    /* Update diffr */
    HeUpdateMark (hn, HE_UPD_DRAW);

    /* Fin succes */
    return hn;
}


/*
 * Set
*/

void HeSetCanvasEventProc (He_node *hn, He_event_proc proc)
{
    if (HeNotCanvas(hn)) return;   
    hn->OBJ_UNION->event_proc = proc;
}

void HeSetCanvasResizeProc (He_node *hn, He_resize_proc proc)
{
    if (HeNotCanvas(hn)) return;   
    hn->OBJ_UNION->resize_proc = proc;
}

void HeSetCanvasRepaintProc (He_node *hn, He_repaint_proc proc)
{
    if (HeNotCanvas(hn)) return;   
    hn->OBJ_UNION->repaint_proc = proc;
}

void HeSetCanvasDBuf (He_node *hn, int val)
{
    OBJ_TYPE *ob;

    if (HeNotCanvas(hn)) return;
    ob = hn->OBJ_UNION;

    if (HeCanvasDBufInit () <= 0) return;

    val = val ? 1 : 0;
    if (ob->dbuf == val) return;

    ob->dbuf = val;
    if (ob->dbuf)
         HeCanvasDBufAlloc (hn);
    else HeCanvasDBufFree (hn);
}


/*
 * Get
*/

He_event_proc HeGetCanvasEventProc (He_node *hn)
{
    if (HeNotCanvas(hn)) return NULL;
    return hn->OBJ_UNION->event_proc;
}

He_resize_proc HeGetCanvasResizeProc (He_node *hn)
{
    if (HeNotCanvas(hn)) return NULL;
    return hn->OBJ_UNION->resize_proc;
}

He_repaint_proc HeGetCanvasRepaintProc (He_node *hn)
{
    if (HeNotCanvas(hn)) return NULL;
    return hn->OBJ_UNION->repaint_proc;
}

int HeGetCanvasDBuf (He_node *hn)
{
    if (HeNotCanvas(hn)) return FALSE;
    return hn->OBJ_UNION->dbuf;
}


/*-------------------- M E T H O D E S - P R I V E E S ------------------------
 *
 * On ne teste pas si hn == NULL.
*/

/*
 * Initialisation du module ; appel par HeCreateCanvas().
*/

void HeCanvasAutoInit ()
{
    static int did_init = FALSE;
    
    if (did_init) return;
    
    HeMethodDefault (&he_meth_canvas, HE_NAT_CANVAS);
    he_meth_canvas.props   = HE_PM_WIN_OWNER;
    he_meth_canvas.destroy = HeCanvasDestroy;
    he_meth_canvas.update  = HeCanvasUpdate;
    he_meth_canvas.event   = HeCanvasEvent;
    he_meth_canvas.tip     = HeCanvasStoreTip;
    
    did_init = TRUE;
}


/*
 * Destruction d'un canvas ; appele' par HeDestroyRec().
*/

void HeCanvasDestroy (He_node *hn)
{
    OBJ_TYPE *ob = hn->OBJ_UNION;

    HeCanvasDBufFree (hn);

    HeWinListRemove (hn);
    XDestroyWindow (he_display, hn->win);
    
    HeTipDestroyInfo (&ob->tip_info);

    free (ob);
}


/*
 * M-A-J geometrie et affichage ; appele' par HeUpdateFlush().
*/

void HeCanvasUpdate (He_node *hn, He_upd_info *ui)
{
    HeUpdateWinGeom (hn, ui);
        
    if (HE_BIT_ISSET (ui->mask, HE_UPD_BORDER)) {
	XSetWindowBorderWidth (he_display, hn->win, hn->win_border);
    }

    if (HE_BIT_ISSET (ui->mask, HE_UPD_DRAW)) {
	if (HE_BIT_ISSET (hn->status, HE_STA_SHOW)) 
	     XMapWindow   (he_display, hn->win);
	else XUnmapWindow (he_display, hn->win);
    }
    if (HE_BIT_ISSET (ui->mask, HE_UPD_EXPOSE)) {
	HeSendExpose (hn->win);
    }
}


/*
 * Gestion evenements X ; appele' par HeEventDispatch().
 *
 * Evenements a reporter a l'event_proc :
 *     on passe directement la structure He_event.
 *
 * Renvoie -1 si erreur.
*/

int HeCanvasEvent (He_node *hn, He_event *hev)
{
    OBJ_TYPE *ob = hn->OBJ_UNION;

    HeTipEvent (hn, hev);
    
    switch (hev->type) {

	case EnterNotify :
	case LeaveNotify : {
        
            /* Emprunte ou restitue le focus clavier
               (sauf en cas de drag, dtect avec sb != 0) */
            if (hev->sb == 0)
		HeFocusBorrow (hn, hev->type == EnterNotify); 
                
            /* Appel de l'EventProc */
	    if (ob->event_proc != NULL)	(ob->event_proc)(hn, hev);
	} break;
        
	case KeyPress :	
	case KeyRelease : {
            if (HE_BIT_ISSET (hn->status, HE_STA_FOCUS_OWN)) {
		/* Appel de l'EventProc */
	        if (ob->event_proc != NULL) (ob->event_proc)(hn, hev);
            } else {
                HeSendEventToFocus (hn, hev);
            }
	} break;
        
	case ButtonPress :
	case ButtonRelease :
	case MotionNotify : {
            /* Appel de l'EventProc */
	    if (ob->event_proc != NULL)	(ob->event_proc)(hn, hev);
	} break;


	/* Changement de la geometrie d'un Window 
	** On met a jour les dimensions.
	*/
	case ConfigureNotify : {

	    XConfigureEvent *ev = &hev->xev->xconfigure;

	    if (hn->xm != ev->width || hn->ym != ev->height) {
		hn->xm = ev->width; hn->ym = ev->height; 
	    } 
	} break;


	/* Expose d'un Window (demande de refresh) 
	**
	** On appelle la resize_proc si necessaire, puis la repaint_proc.
        ** Ceci garantit a l'utilisateur que la resize_proc precede toujours
        ** la repaint_proc lors d'un changement de dimensions.
	*/
	case Expose : {

	    /* On teste si les dimensions ont change */
	    if (ob->old_xm != hn->xm || ob->old_ym != hn->ym) {
            
                /* M-a-j pour prochain test */
                ob->old_xm = hn->xm; ob->old_ym = hn->ym;
                
		if (ob->resize_proc != NULL)
		    (ob->resize_proc)(hn, hn->xm, hn->ym);
            }
            
            /* repaint_proc appele APRES resize_proc */
	    if (ob->repaint_proc != NULL) {
		(ob->repaint_proc)(hn, ob->dbuf ? ob->back_buf : hev->win);
		HeCanvasDBufSwap (hn);
	    }
	} break;

    } /* switch */

    return 0;
}


/*
 * Mthode renvoyant &tip.
*/
He_tip_info *HeCanvasStoreTip (He_node *hn)
{
    return &hn->OBJ_UNION->tip_info;
}


/*-------------------- F O N C T I O N S - P R I V E E S ---------------------*/

/*
 * Gestion du double buffer pour afficher sans clignottement.
*/

int HeCanvasDBufInit ()
{
#ifdef HE_USE_XEXT
    static int init_dbuf = 0;

    /* Init de l'extension double buffer */
    if (init_dbuf == 0) {
	int m1, m2;
	if (XdbeQueryExtension (he_display, &m1, &m2) == 0) {
	    HeError ("Can't init double buffer\n");
	    init_dbuf = -1;
	} else init_dbuf = 1;
    }
    return init_dbuf;
#else
    return -1;
#endif
}

void HeCanvasDBufAlloc (He_node *hn)
{
#ifdef HE_USE_XEXT
    OBJ_TYPE *ob = hn->OBJ_UNION;
    if (ob->dbuf && ob->back_buf == None)
	ob->back_buf = XdbeAllocateBackBufferName (he_display, 
			    hn->win, XdbeUndefined);
#endif
}

void HeCanvasDBufFree (He_node *hn)
{
#ifdef HE_USE_XEXT
    OBJ_TYPE *ob = hn->OBJ_UNION;
    if (ob->dbuf && ob->back_buf != None) {
	XdbeDeallocateBackBufferName (he_display, ob->back_buf);
	ob->back_buf = None;
    }
#endif
}

void HeCanvasDBufSwap (He_node *hn)
{
#ifdef HE_USE_XEXT
    OBJ_TYPE *ob = hn->OBJ_UNION;
    XdbeSwapInfo swap_info[] = { { hn->win, XdbeUndefined } };
    if (ob->dbuf)
        XdbeSwapBuffers (he_display, swap_info, 1);
#endif
}


/*
 *  Teste nature du noeud ; renvoie TRUE si erreur. 
*/

int HeNotCanvas (He_node *hn)
{
    if (hn == NULL || hn->meth != &he_meth_canvas) {
	HeError ("Canvas expected\n");
        return TRUE;
    }
    return FALSE;
}


