/*
 * The Helium Toolkit
 *
 * Copyright (C) 1996-2000 Edouard Thiel <Edouard.Thiel@lif.univ-mrs.fr>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License; see http://www.lif-sud.univ-mrs.fr/~thiel/helium/LICENSE
*/

/*
 * gui/dialog.c - 08/07/1999
 *
 * 
*/

#include <helium.h>


static He_node *dialog_frame = NULL;
static He_dialog_proc dialog_proc = NULL;
static void *dialog_client_data = NULL;


/*------------------- I N T E R F A C E - P U B L I Q U E --------------------*/

/*
 * Creation et affichage d'une fenetre de dialogue.
 * Une seule fenetre de dialogue peut etre affichee a la fois.
 * Un nouvel appel  HeSimpleDialog remplace la fentre precedente.
 *
 * Syntaxe :
 *    HeSimpleDialog (
 *        HE_DIALOG_BELL,
 *        HE_DIALOG_TITLE,   "Le titre",
 *        HE_DIALOG_MESSAGE, "Attention, le fichier",
 *        HE_DIALOG_QUOTED,  file_name,
 *        HE_DIALOG_MESSAGE, "ne peut etre ouvert",
 *        HE_DIALOG_BUTTOK,  "Ok"
 *        HE_DIALOG_BUTTON,  "Reessayer"
 *        HE_DIALOG_BUTTON,  "Annuler"
 *        HE_DIALOG_PROC,    my_dialog_proc
 *        HE_DIALOG_DATA,    my_client_data,
 *        0);
*/

void HeSimpleDialog (int yop, ...)
{
    He_node *panel, *v_mess_widget[100], *v_butt_widget[100], *rule;
    va_list ap;
    char *bla, *v_mess_str[100], *v_butt_str[100] ;
    int ok = TRUE, x, i, focus_button = -1,
         n_mess = 0, n_butt = 0, v_mess_quoted[100];
    
    /* Un seul dialog a la fois : le nouveau remplace l'ancien */
    HeDialogRAZ();
    
    dialog_frame = HeCreateFrame();
    HeSetFrameLabel (dialog_frame, "");
    HeTransientFrame (dialog_frame);
    panel = HeCreatePanel (dialog_frame);
    HeSetPanelKbdProc (panel, HeDialogKbdProc);
    HeSetPanelXAlign (panel, 20);
    HeSetPanelYAlign (panel, 10);
    
    /* Debut lecture arguments */
    va_start (ap, yop);
    
    while (yop) {

	switch (yop) {

 	  case HE_DIALOG_BELL :
            XBell (he_display, 80);
            break;
 	  case HE_DIALOG_TITLE :  
            HeSetFrameLabel (dialog_frame, va_arg (ap, char*));
            break;
 	  case HE_DIALOG_MESSAGE :
	    bla = va_arg (ap, char*);
            if (n_mess < 100) { 
                v_mess_str[n_mess] = bla; 
                v_mess_quoted[n_mess++] = FALSE;
            }
            break;
 	  case HE_DIALOG_QUOTED :
	    bla = va_arg (ap, char*);
            if (n_mess < 100) { 
                v_mess_str[n_mess] = bla; 
                v_mess_quoted[n_mess++] = TRUE;
            }
            break;
  	  case HE_DIALOG_BUTTOK :
            if (n_butt < 100) focus_button = n_butt;
  	  case HE_DIALOG_BUTTON :
	    bla = va_arg (ap, char*);
            if (n_butt < 100) v_butt_str[n_butt++] = bla;
            break;
  	  case HE_DIALOG_PROC :
            dialog_proc = va_arg (ap, He_dialog_proc);
            break;
  	  case HE_DIALOG_DATA :
            dialog_client_data = va_arg (ap, void*);
            break;
	  default :
	    HeError ("HeSimpleDialog: invalid argument\n");
            ok = FALSE;
	    break;
	}
	yop = ok ? va_arg (ap, int) : 0;
    }
    
    /* Fin lecture arguments */
    va_end (ap);


    /* Creation messages */
    HeSetPanelLayout (panel, HE_VERTICAL);
    HeSetPanelYGap (panel, 0);
    for (i = 0; i < n_mess; i++)
      if (v_mess_quoted[i]) {
          char *buf = malloc (strlen(v_mess_str[i])+3);
          sprintf (buf, "\"%s\"", v_mess_str[i]);
          v_mess_widget[i] = HeCreateMessageP (panel, buf, FALSE);
          free(buf);
      } else
          v_mess_widget[i] = HeCreateMessageP (panel, v_mess_str[i], FALSE);

    /* Creation separateur */
    rule = HeCreateRule (panel);
    
    /* Creation boutons */
    HeSetPanelLayout (panel, HE_LINE_FEED);
    for (i = 0; i < n_butt; i++)
	v_butt_widget[i] = HeCreateButtonP (panel,
        	v_butt_str[i], HeDialogButtProc, NULL);
    if (focus_button != -1)
        HeSetFocus (v_butt_widget[focus_button], TRUE);
    
    HeFit (panel);
    
    /* on centre les messages */
    x = HeGetWidth (panel);
    for (i = 0; i < n_mess; i++)
	HeSetX (v_mess_widget[i], (x - HeGetWidth(v_mess_widget[i]))/2);
        
    /* on elargit le separateur */
    HeSetX (rule, 10);
    HeSetWidth (rule, x - 20);
        
    /* on centre les ventuels boutons */
    if (n_butt > 0)			/* bugfix Sylvain Hitier 12/06/2003 */
    {
        x -= HeGetPanelXAlign(panel);
        x -= HeGetX(v_butt_widget[n_butt-1]) + HeGetWidth(v_butt_widget[n_butt-1]);
        x /= 2;
        for (i = 0; i < n_butt; i++)
	    HeSetX (v_butt_widget[i], x + HeGetX(v_butt_widget[i]));
    }
    
    /* on centre la fenetre a l'ecran */
    HeFit (dialog_frame);
    HeSetX (dialog_frame, (he_root_width  - HeGetWidth (dialog_frame))/2);
    HeSetY (dialog_frame, (he_root_height - HeGetHeight(dialog_frame))/2);
    HeSetShow (dialog_frame, TRUE);
}



/*-------------------- F O N C T I O N S - P R I V E E S ---------------------*/

/*
 * Reinitialise les variables globales.
*/

void HeDialogRAZ ()
{
    HeDestroy (dialog_frame);
    dialog_frame = NULL;
    dialog_proc = NULL;
    dialog_client_data = NULL;
}


/*
 * Callback associee au panel.
 *
 * On intercepte Escape pour fermer la fentre.
*/

int HeDialogKbdProc (He_node *hn, He_event *hev)
{
    if (hev->type == KeyPress && hev->sym == XK_Escape) {
    
	/* Destruction et RAZ */
	HeDialogRAZ ();
        
	/* Interception */
	return FALSE;
    }

    return TRUE;
}


/*
 * Callback associee a chaque bouton.
 *
 * La fenetre de dialogue est detruite juste avant l'appel de la dialog_proc ;
 * de la sorte, si un nouvel appel a HeSimpleDialog a lieu dans la dialog_proc,
 * une nouvelle fenetre est cree.
*/

void HeDialogButtProc (He_node *hn)
{
    He_dialog_proc proc = dialog_proc;
    char *name = HeGetButtonLabel (hn);
    void *client_data = dialog_client_data;
    
    /* Destruction et RAZ */
    HeDialogRAZ ();

    /* Appel proc */
    if (proc != NULL)
	(proc)(name, client_data);
}


/*
 * Appele' par HeMainLoop pour replacer au premier plan une boite de dialogue
 * qui aurait ete eventuellement recouverte.
*/

void HeDialogOnTop ()
{
    if (dialog_frame)
	XMapRaised (he_display, dialog_frame->win);
}

