/*
 * The Helium Toolkit
 *
 * Copyright (C) 1996-2000 Edouard Thiel <Edouard.Thiel@lif.univ-mrs.fr>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License; see http://www.lif-sud.univ-mrs.fr/~thiel/helium/LICENSE
*/

/*
 * gui/error.c - 29/05/1996
 *
 * 
*/

#include <helium.h>


/*------------------- I N T E R F A C E - P U B L I Q U E --------------------*/

/*
 * Affiche un message d'erreur sur stderr.
 * S'utilise comme printf().
*/

int HeError (const char *format, ...)
{
    va_list ap;
    int res;

    fprintf (stderr, "Helium: ");
    va_start (ap, format);
    res = vfprintf (stderr, format, ap);
    fflush (stderr);
    va_end (ap);

    return res;
}


/*
 * Renvoie le nom de l'erreur d'apres le code.
*/

char *HeGetXErrorName (int error_code)
{
    static char error_msg[80];

    XGetErrorText (he_display, error_code, error_msg, 80);
    return error_msg;
}


/*
 * Renvoie le nom de l'evenement d'apres le code, ou "Unknown".
*/

char *HeGetXEventName (int event_code)
{
    static char *names[] = {	/* cf X11/X.h */
	"",			"",			"KeyPress",
	"KeyRelease",		"ButtonPress",		"ButtonRelease",
	"MotionNotify",		"EnterNotify",		"LeaveNotify",	
	"FocusIn",		"FocusOut",		"KeymapNotify",
	"Expose",		"GraphicsExpose",	"NoExpose",
	"VisibilityNotify",	"CreateNotify",		"DestroyNotify",
	"UnmapNotify",		"MapNotify",		"MapRequest",
	"ReparentNotify",	"ConfigureNotify",	"ConfigureRequest",
	"GravityNotify",	"ResizeRequest",	"CirculateNotify",
	"CirculateRequest",	"PropertyNotify",	"SelectionClear",
	"SelectionRequest",	"SelectionNotify",	"ColormapNotify",
	"ClientMessage",	"MappingNotify",
    };

    return (event_code > 1 && event_code < LASTEvent)
	? names[event_code] : "Unknown" ;
}


/*
 * Renvoie le nom de la requete d'apres le code, ou "Unknown".
*/

char *HeGetXRequestName (int request_code)
{
    static char *names[] = {	/* cf X11/Xproto.h */
	"", "X_CreateWindow", "X_ChangeWindowAttributes", 
	"X_GetWindowAttributes", "X_DestroyWindow", "X_DestroySubwindows",
	"X_ChangeSaveSet", "X_ReparentWindow", "X_MapWindow", "X_MapSubwindows",
	"X_UnmapWindow", "X_UnmapSubwindows", "X_ConfigureWindow",
	"X_CirculateWindow", "X_GetGeometry", "X_QueryTree", "X_InternAtom",
	"X_GetAtomName", "X_ChangeProperty", "X_DeleteProperty",
	"X_GetProperty", "X_ListProperties", "X_SetSelectionOwner",
	"X_GetSelectionOwner", "X_ConvertSelection", "X_SendEvent",
	"X_GrabPointer", "X_UngrabPointer", "X_GrabButton", "X_UngrabButton",
	"X_ChangeActivePointerGrab", "X_GrabKeyboard", "X_UngrabKeyboard",
	"X_GrabKey", "X_UngrabKey", "X_AllowEvents", "X_GrabServer",
	"X_UngrabServer", "X_QueryPointer", "X_GetMotionEvents",
	"X_TranslateCoords", "X_WarpPointer", "X_SetInputFocus",
	"X_GetInputFocus", "X_QueryKeymap", "X_OpenFont", "X_CloseFont",
	"X_QueryFont", "X_QueryTextExtents", "X_ListFonts",
	"X_ListFontsWithInfo", "X_SetFontPath", "X_GetFontPath",
	"X_CreatePixmap", "X_FreePixmap", "X_CreateGC", "X_ChangeGC", 
	"X_CopyGC", "X_SetDashes", "X_SetClipRectangles", "X_FreeGC",
	"X_ClearArea", "X_CopyArea", "X_CopyPlane", "X_PolyPoint", "X_PolyLine",
	"X_PolySegment", "X_PolyRectangle", "X_PolyArc", "X_FillPoly",
	"X_PolyFillRectangle", "X_PolyFillArc", "X_PutImage", "X_GetImage",
	"X_PolyText8", "X_PolyText16", "X_ImageText8", "X_ImageText16",
	"X_CreateColormap", "X_FreeColormap", "X_CopyColormapAndFree",
	"X_InstallColormap", "X_UninstallColormap", "X_ListInstalledColormaps",
	"X_AllocColor", "X_AllocNamedColor", "X_AllocColorCells",
	"X_AllocColorPlanes", "X_FreeColors", "X_StoreColors",
	"X_StoreNamedColor", "X_QueryColors", "X_LookupColor", "X_CreateCursor",
	"X_CreateGlyphCursor", "X_FreeCursor", "X_RecolorCursor",
	"X_QueryBestSize", "X_QueryExtension", "X_ListExtensions",
	"X_ChangeKeyboardMapping", "X_GetKeyboardMapping", 
	"X_ChangeKeyboardControl", "X_GetKeyboardControl", "X_Bell",
	"X_ChangePointerControl", "X_GetPointerControl", "X_SetScreenSaver",
	"X_GetScreenSaver", "X_ChangeHosts", "X_ListHosts", 
	"X_SetAccessControl", "X_SetCloseDownMode", "X_KillClient",
	"X_RotateProperties", "X_ForceScreenSaver", "X_SetPointerMapping",
	"X_GetPointerMapping", "X_SetModifierMapping", "X_GetModifierMapping",
	"Unknown", "Unknown", "Unknown", "Unknown", "Unknown", "Unknown",
	"Unknown", "X_NoOperation"
    };

    return (request_code > 0 && request_code < 128)
	? names[request_code] : "Unknown" ;
}


/*-------------------- F O N C T I O N S - P R I V E E S ---------------------*/

/*
 * Nouveau ErrorHandler pour X11, qui affiche les informations de l'erreur
 * sans quitter l'application.
 *
 * Silencieux pour : BadWindow, BadAtom
 *
 * Installation dans HeInit() : XSetErrorHandler(HeNonFatalXErrorHandler);
*/

int HeNonFatalXErrorHandler (Display *display, XErrorEvent *err)
{
#   ifndef DEBUG
    if (err->error_code == BadWindow ||
        err->error_code == BadAtom ) return 0;
#   endif

    HeError ("X request failed: %s\n"
	     "    Major opcode: %d (%s)\n"
	     "    Ressource id: 0x%x    Serial number: %d\n",
	HeGetXErrorName (err->error_code), 
	err->request_code, HeGetXRequestName (err->request_code),
	err->resourceid, err->serial);

    return 0;  /* valeur ignore'e par XSetErrorHandler() */
}







