/*
 * The Helium Toolkit
 *
 * Copyright (C) 1996-2000 Edouard Thiel <Edouard.Thiel@lif.univ-mrs.fr>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License; see http://www.lif-sud.univ-mrs.fr/~thiel/helium/LICENSE
*/

/*
 * gui/pan_arrow.c - 18/05/2000
 *
 * 
*/

#include <helium.h>

/* Type et union du package 
*/
#define  OBJ_TYPE	Obj_panel_arrow
#define  OBJ_UNION	obj.arrow

static He_method he_meth_arrow;


/*------------------- I N T E R F A C E - P U B L I Q U E --------------------*/

/*
 *  Cration d'un panel_arrow.
 *  Renvoie le noeud ou NULL.
*/

He_node *HeCreateArrow (He_node *owner)
{
    He_node *hn;
    OBJ_TYPE *ob;

    HeArrowAutoInit();

    /* On cree le noeud */
    hn = HeBaseCreate (owner, &he_meth_arrow);
    if (hn == NULL) return NULL;

    /* On cree l'objet */
    ob = hn->OBJ_UNION = malloc (sizeof (OBJ_TYPE));
    if (ob == NULL) {
	HeError ("HeCreateArrow: malloc error\n");
	return NULL;
    }

    /* Init des champs du noeud */    
    hn->xm = hn->ym = 0;		/* Inhibe le 1er HeUpdateDrawOldBg */
    hn->win = hn->owner->win;

    /* init des champs de l'objet */
    ob->current = 0;
    ob->slow = 1;
    ob->fast = 0;
    ob->layout = HE_HORIZONTAL;
    ob->notify_proc = NULL;

    /* Positionne l'objet par rapport a PANEL_LAYOUT */
    HePosPanelItem (hn);

    /* Fixe taille par defaut, provoque HE_UPD_ANYDIM | HE_UPD_DRAW */
    HeArrowFit (hn);

    /* Fin succes */
    return hn;
}


/*
 * Set
*/

void HeSetArrowCurrentValue (He_node *hn, int current)
{
    if (HeNotArrow(hn)) return;
    hn->OBJ_UNION->current = current;
    if (HE_BIT_ISSET (hn->status, HE_STA_AUTOFIT)) HeArrowFit (hn);
}

void HeSetArrowSlowValue (He_node *hn, int slow)
{
    if (HeNotArrow(hn)) return;
    hn->OBJ_UNION->slow = slow;
    if (HE_BIT_ISSET (hn->status, HE_STA_AUTOFIT)) HeArrowFit (hn);
}

void HeSetArrowFastValue (He_node *hn, int fast)
{
    if (HeNotArrow(hn)) return;
    hn->OBJ_UNION->fast = fast;
    if (HE_BIT_ISSET (hn->status, HE_STA_AUTOFIT)) HeArrowFit (hn);
}

void HeSetArrowLayout (He_node *hn, int layout)
{
    if (HeNotArrow(hn)) return;
    hn->OBJ_UNION->layout = layout;
    if (HE_BIT_ISSET (hn->status, HE_STA_AUTOFIT)) HeArrowFit (hn);
}

void HeSetArrowNotifyProc (He_node *hn, He_notify_proc proc)
{
    if (HeNotArrow(hn)) return;
    hn->OBJ_UNION->notify_proc = proc;
}


/*
 * Get
*/

int HeGetArrowCurrentValue (He_node *hn)
{
    if (HeNotArrow(hn)) return 0;
    return hn->OBJ_UNION->current;
}

int HeGetArrowSlowValue (He_node *hn)
{
    if (HeNotArrow(hn)) return 0;
    return hn->OBJ_UNION->slow;
}

int HeGetArrowFastValue (He_node *hn)
{
    if (HeNotArrow(hn)) return 0;
    return hn->OBJ_UNION->fast;
}

int HeGetArrowLayout (He_node *hn)
{
    if (HeNotArrow(hn)) return 0;
    return hn->OBJ_UNION->layout;
}

He_notify_proc HeGetArrowNotifyProc (He_node *hn)
{
    if (HeNotArrow(hn)) return NULL;
    return hn->OBJ_UNION->notify_proc;
}


/*-------------------- M E T H O D E S - P R I V E E S ------------------------
 *
 * On ne teste pas si hn == NULL.
*/

/*
 * Initialisation du module ; appel par HeCreateArrow().
*/

void HeArrowAutoInit ()
{
    static int did_init = FALSE;
    
    if (did_init) return;
    
    HeMethodDefault (&he_meth_arrow, HE_NAT_ARROW);
    he_meth_arrow.props   = HE_PM_ACCEPT_FOCUS;
    he_meth_arrow.destroy = HeArrowDestroy;
    he_meth_arrow.fit     = HeArrowFit;
    he_meth_arrow.update  = HeArrowUpdate;
    he_meth_arrow.event   = HeArrowEvent;
    he_meth_arrow.draw    = HeArrowDraw;
    
    did_init = TRUE;
}


/*
 * Destruction d'un panel_arrow ; appele' par HeDestroyRec().
*/

void HeArrowDestroy (He_node *hn)
{
    OBJ_TYPE *ob = hn->OBJ_UNION;

    free (ob);
}


/*
 * Ajuste la taille du panel_arrow en fonction de son contenu.
*/

void HeArrowFit (He_node *hn)
{
    int xm, ym;
    
    if (HeNotArrow(hn)) return;
    
    HeArrowSize (hn, HE_DECO_TW, HE_DECO_TH, &xm, &ym);
    HeSetWidth (hn, xm);
    HeSetHeight(hn, ym);
    HeUpdateMark (hn, HE_UPD_DRAW);
}


/*
 * M-A-J geometrie et affichage ; appele' par HeUpdateFlush().
*/

void HeArrowUpdate (He_node *hn, He_upd_info *ui)
{
    HeUpdateDrawOldBg (hn, ui);

    /* Se'curite' pour que le nom ne de'passe pas le widget */
    if (HE_BIT_ISSET (ui->mask, HE_UPD_SIZE)) {
	int x, y;

	HeArrowSize (hn, HE_DECO_TM, HE_DECO_TM, &x, &y);
	if (hn->xm < x) hn->xm = x;
	if (hn->ym < y) hn->ym = y;
    }

    if (HE_BIT_ISSET (ui->mask, HE_UPD_ANYDIM | HE_UPD_DRAW | HE_UPD_FOCUS)) 
	HeArrowDraw (hn);
}


/*
 * Methode pour gerer les evenements.
 * Renvoie -1 si erreur.
 *
 * Appele' par HePanelEvent()
*/

int HeArrowEvent (He_node *hn, He_event *hev)
{
    OBJ_TYPE *ob = hn->OBJ_UNION;
    int redraw = FALSE;

#ifdef DEBUG
    printf ("    arrow %s %s\n", ob->label, HeGetXEventName (hev->type)); 
#endif

    switch (hev->type) {

	case KeyPress :
            if (!HE_BIT_ISSET (hn->status, HE_STA_ACTIVE)) break;
            if (!(hev->sym == XK_Return ||
                  hev->sym == XK_KP_Enter ||
                  hev->sym == XK_ISO_Enter)) break;
            if (HE_BIT_ISSET (hn->status, HE_STA_PUSHED)) break;
	    HE_BIT_SET (hn->status, HE_STA_PUSHED);
	    redraw = TRUE;
	    break; 
            
	case ButtonPress :
            if (!HE_BIT_ISSET (hn->status, HE_STA_ACTIVE)) break;
            if (HE_BIT_ISSET (hn->status, HE_STA_PUSHED)) break;
            if (HE_BIT_ISSET (hev->xev->xbutton.state, ShiftMask))
		 HeSetFocus (hn, TRUE); 
	    else HE_BIT_SET (hn->status, HE_STA_PUSHED);
	    redraw = TRUE;
	    break; 

	case KeyRelease :
            if (!(hev->sym == XK_Return ||
                  hev->sym == XK_KP_Enter ||
                  hev->sym == XK_ISO_Enter)) break;
	case ButtonRelease :
            if (!HE_BIT_ISSET (hn->status, HE_STA_ACTIVE)) break;
            if (!HE_BIT_ISSET (hn->status, HE_STA_PUSHED)) break;
	    HE_BIT_CLR (hn->status, HE_STA_PUSHED);
	    redraw = TRUE;
	    /* Appelle la notify_proc */
	    if (ob->notify_proc != NULL)
		(ob->notify_proc)(hn);
	    break; 

	case MotionNotify :
	    break;	

	case EnterNotify : 
	    HE_BIT_SET (hn->status, HE_STA_MOUSE_IN);
	    redraw = TRUE;
	    break; 

	case LeaveNotify : 
	    HE_BIT_CLR (hn->status, HE_STA_PUSHED);
            HE_BIT_CLR (hn->status, HE_STA_MOUSE_IN);
	    redraw = TRUE;
	    break; 

	case Expose :
	    redraw = TRUE;
	    break; 

    } /* switch */

    if (redraw) HeArrowDraw (hn);

    return 0;
}


/*
 * Dessin du widget.
 *
*/

void HeArrowDraw (He_node *hn)
{
#if 0
    OBJ_TYPE *ob = hn->OBJ_UNION;
    Window win = hn->win;
    int show     = HE_BIT_ISSET (hn->status, HE_STA_SHOW),
        pushed   = HE_BIT_ISSET (hn->status, HE_STA_PUSHED),
        mouse_in = HE_BIT_ISSET (hn->status, HE_STA_MOUSE_IN),
        active   = HE_BIT_ISSET (hn->status, HE_STA_ACTIVE),
        focus    = HE_BIT_ISSET (hn->status, HE_STA_FOCUS_IN),
        survol   = active && (mouse_in || pushed),
        c_bg     = (active && pushed && ob->xi == NULL) ? 
                       he_col_rev_bg : he_col_bg,
        c1_ext   = (pushed) ? he_col_dark_bd : he_col_light_bd,
        c2_ext   = (pushed) ? he_col_light_bd : he_col_dark_bd,
        c1_int   = (survol) ? he_col_dark_bd : c_bg,
        c2_int   = (survol) ? he_col_light_bd : c_bg,
        text     = (ob->label && ob->label[0]),
        y1       = 0;

/* printf ("draw arrow %s\n", ob->label); */

    /* Dessin du fond */
    HeDrawBg (hn, (show) ? c_bg : he_col_bg);
    if (!show) return;

    /* Dessin bord ou focus */
    if (focus)
        HeDrawRect3DC (win, hn->xb, hn->yb, hn->xm-1, hn->ym-1,
	    he_black, he_black );
    else
	HeDrawRect3DC (win, hn->xb, hn->yb, hn->xm-1, hn->ym-1,
	    c1_ext, c2_ext);

    /* Survole' par la souris */
    if (survol)
	HeDrawRect3D (win, hn->xb+1, hn->yb+1, hn->xm-3, hn->ym-3,
	    c1_int, c2_int);
    
    /* Calcul coordonne y1 */
    if (ob->xi && text) 
        y1 = hn->yb + (hn->ym - ob->xi->height - he_font_height) 
                      * HE_DECO_TW / (HE_DECO_TW + 2*HE_DECO_TH);
    
    /* Image */
    if (ob->xi) {
        int xb = hn->xb + (hn->xm - ob->xi->width) / 2,
            yb = (text) ? y1 : hn->yb + (hn->ym - ob->xi->height) / 2;
	HePutXi (hn->win, he_gc_draw, ob->xi, xb, yb);
    }
    
    /* Nom */
    if (text) {
        int yb = (ob->xi) ? y1 + ob->xi->height : hn->yb,
            ym = hn->ym - (yb - hn->yb);
    
        XSetForeground (he_display, he_gc_draw,
            (!active) ? he_col_light_bd :
            (pushed) ? he_col_rev_ink : he_col_ink );
        HeDrawStringCenterRect(win, he_gc_draw, he_normal_font,
            hn->xb, yb, hn->xm, ym, ob->label);
    }

#endif
}


/*-------------------- F O N C T I O N S - P R I V E E S ---------------------*/

/*
 * Calcule la taille xm,ym du widget selon label et xi
 *
*/
void HeArrowSize (He_node *hn, int deco_x, int deco_y, int *xm, int *ym)
{
#if 0
    OBJ_TYPE *ob = hn->OBJ_UNION;
    int text = (ob->label && ob->label[0]);
    
    if (text) {
	HeDimString (he_normal_font, ob->label, xm, ym);
        if (ob->xi) {
            if (*xm < ob->xi->width) *xm = ob->xi->width;
	    *xm += deco_x;
            *ym += ob->xi->height + deco_y + deco_x/2;       
        } else {
	    *xm += deco_x;
            *ym += deco_y;
        }
    } else {
        if (ob->xi) {
	     *xm = ob->xi->width  + deco_x; 
             *ym = ob->xi->height + deco_x; /* bord large */
        } else {
	     *xm = deco_x; 
             *ym = deco_y + he_font_height; 
        }
    }
#endif
*xm = 50; *ym = 20;
}


/*
 *  Teste nature du noeud ; renvoie TRUE si erreur. 
*/

int HeNotArrow (He_node *hn)
{
    if (hn == NULL || hn->meth != &he_meth_arrow) {
	HeError ("Arrow expected\n");
        return TRUE;
    }
    return FALSE;
}

