/*
 * The Helium Toolkit
 *
 * Copyright (C) 1996-2000 Edouard Thiel <Edouard.Thiel@lif.univ-mrs.fr>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License; see http://www.lif-sud.univ-mrs.fr/~thiel/helium/LICENSE
*/

/*
 * gui/pan_button.c - 30/06/1999
 *
 * 
*/

#include <helium.h>

/* Type et union du package 
*/
#define  OBJ_TYPE	Obj_panel_button
#define  OBJ_UNION	obj.button

static He_method he_meth_button;


/*------------------- I N T E R F A C E - P U B L I Q U E --------------------*/

/*
 *  Cration d'un panel_button.
 *  Renvoie le noeud ou NULL.
*/

He_node *HeCreateButton (He_node *owner)
{
    He_node *hn;
    OBJ_TYPE *ob;

    HeButtonAutoInit();

    /* On cree le noeud */
    hn = HeBaseCreate (owner, &he_meth_button);
    if (hn == NULL) return NULL;

    /* On cree l'objet */
    ob = hn->OBJ_UNION = malloc (sizeof (OBJ_TYPE));
    if (ob == NULL) {
	HeError ("HeCreateButton: malloc error\n");
	return NULL;
    }

    /* Init des champs du noeud */    
    hn->xm = hn->ym = 0;		/* Inhibe le 1er HeUpdateDrawOldBg */
    hn->win = hn->owner->win;

    /* init des champs de l'objet */
    ob->label = NULL;
    ob->xi = NULL;
    ob->notify_proc = NULL;
    HeTipInitInfo (&ob->tip_info);

    /* Positionne l'objet par rapport a PANEL_LAYOUT */
    HePosPanelItem (hn);

    /* Fixe taille par defaut, provoque HE_UPD_ANYDIM | HE_UPD_DRAW */
    HeButtonFit (hn);

    /* Fin succes */
    return hn;
}


/*
 * Creation d'un bouton parametre'
*/

He_node *HeCreateButtonP (He_node *owner, const char *label, 
                          He_notify_proc proc, void *client_data)
{
    He_node *tmp = HeCreateButton (owner);
    HeSetButtonLabel (tmp, label);
    HeSetButtonNotifyProc (tmp, proc);
    HeSetClientData (tmp, client_data);
    return tmp;
}


/*
 * Set
*/

void HeSetButtonLabel (He_node *hn, const char *label)
{
    OBJ_TYPE *ob;
    
    if (HeNotButton(hn)) return;
    ob = hn->OBJ_UNION;
    
    free (ob->label);
    ob->label = (label == NULL) ? NULL : strdup(label);
    
    if (HE_BIT_ISSET (hn->status, HE_STA_AUTOFIT)) HeButtonFit (hn);
}

void HeSetButtonXImage (He_node *hn, XImage *xi)
{
    if (HeNotButton(hn)) return;
    hn->OBJ_UNION->xi = xi;
    if (HE_BIT_ISSET (hn->status, HE_STA_AUTOFIT)) HeButtonFit (hn);
}

void HeSetButtonNotifyProc (He_node *hn, He_notify_proc proc)
{
    if (HeNotButton(hn)) return;
    hn->OBJ_UNION->notify_proc = proc;
}


/*
 * Get
*/

char *HeGetButtonLabel (He_node *hn)
{
    if (HeNotButton(hn)) return he_empty_str;
    return hn->OBJ_UNION->label ? hn->OBJ_UNION->label : he_empty_str;
}

XImage *HeGetButtonXImage (He_node *hn)
{
    if (HeNotButton(hn)) return NULL;
    return hn->OBJ_UNION->xi;
}

He_notify_proc HeGetButtonNotifyProc (He_node *hn)
{
    if (HeNotButton(hn)) return NULL;
    return hn->OBJ_UNION->notify_proc;
}


/*-------------------- M E T H O D E S - P R I V E E S ------------------------
 *
 * On ne teste pas si hn == NULL.
*/

/*
 * Initialisation du module ; appel par HeCreateButton().
*/

void HeButtonAutoInit ()
{
    static int did_init = FALSE;
    
    if (did_init) return;
    
    HeMethodDefault (&he_meth_button, HE_NAT_BUTTON);
    he_meth_button.props   = HE_PM_ACCEPT_FOCUS;
    he_meth_button.destroy = HeButtonDestroy;
    he_meth_button.fit     = HeButtonFit;
    he_meth_button.update  = HeButtonUpdate;
    he_meth_button.event   = HeButtonEvent;
    he_meth_button.draw    = HeButtonDraw;
    he_meth_button.tip     = HeButtonStoreTip;

    did_init = TRUE;
}


/*
 * Destruction d'un panel_button ; appele' par HeDestroyRec().
*/

void HeButtonDestroy (He_node *hn)
{
    OBJ_TYPE *ob = hn->OBJ_UNION;

    free (ob->label);
    HeTipDestroyInfo (&ob->tip_info);

    free (ob);
}


/*
 * Ajuste la taille du panel_button en fonction de son contenu.
*/

void HeButtonFit (He_node *hn)
{
    int xm, ym;
    
    if (HeNotButton(hn)) return;
    
    HeButtonSize (hn, HE_DECO_TW, HE_DECO_TH, &xm, &ym);
    HeSetWidth (hn, xm);
    HeSetHeight(hn, ym);
    HeUpdateMark (hn, HE_UPD_DRAW);
}


/*
 * M-A-J geometrie et affichage ; appele' par HeUpdateFlush().
*/

void HeButtonUpdate (He_node *hn, He_upd_info *ui)
{
    HeUpdateDrawOldBg (hn, ui);

    /* Se'curite' pour que le nom ne de'passe pas le bouton */
    if (HE_BIT_ISSET (ui->mask, HE_UPD_SIZE)) {
	int x, y;

	HeButtonSize (hn, HE_DECO_TM, HE_DECO_TM, &x, &y);
	if (hn->xm < x) hn->xm = x;
	if (hn->ym < y) hn->ym = y;
    }

    if (HE_BIT_ISSET (ui->mask, HE_UPD_ANYDIM | HE_UPD_DRAW | HE_UPD_FOCUS)) 
	HeButtonDraw (hn);
}


/*
 * Methode pour gerer les evenements.
 * Renvoie -1 si erreur.
 *
 * Appele' par HePanelEvent()
*/

int HeButtonEvent (He_node *hn, He_event *hev)
{
    OBJ_TYPE *ob = hn->OBJ_UNION;
    int redraw = FALSE;

#ifdef DEBUG
    printf ("    button %s %s\n", ob->label, HeGetXEventName (hev->type)); 
#endif

    HeTipEvent (hn, hev);
    
    switch (hev->type) {

	case KeyPress :
            if (!HE_BIT_ISSET (hn->status, HE_STA_ACTIVE)) break;
            if (!(hev->sym == XK_Return ||
                  hev->sym == XK_KP_Enter ||
                  hev->sym == XK_ISO_Enter)) break;
            if (HE_BIT_ISSET (hn->status, HE_STA_PUSHED)) break;
	    HE_BIT_SET (hn->status, HE_STA_PUSHED);
	    redraw = TRUE;
	    break; 
            
	case ButtonPress :
            if (!HE_BIT_ISSET (hn->status, HE_STA_ACTIVE)) break;
            if (HE_BIT_ISSET (hn->status, HE_STA_PUSHED)) break;
            if (HE_BIT_ISSET (hev->xev->xbutton.state, ShiftMask))
		 HeSetFocus (hn, TRUE); 
	    else HE_BIT_SET (hn->status, HE_STA_PUSHED);
	    redraw = TRUE;
	    break; 

	case KeyRelease :
            if (!(hev->sym == XK_Return ||
                  hev->sym == XK_KP_Enter ||
                  hev->sym == XK_ISO_Enter)) break;
	case ButtonRelease :
            if (!HE_BIT_ISSET (hn->status, HE_STA_ACTIVE)) break;
            if (!HE_BIT_ISSET (hn->status, HE_STA_PUSHED)) break;
	    HE_BIT_CLR (hn->status, HE_STA_PUSHED);
	    redraw = TRUE;
	    /* Appelle la notify_proc */
	    if (ob->notify_proc != NULL)
		(ob->notify_proc)(hn);
	    break; 

	case MotionNotify :
	    break;	

	case EnterNotify : 
	    HE_BIT_SET (hn->status, HE_STA_MOUSE_IN);
	    redraw = TRUE;
	    break; 

	case LeaveNotify : 
	    HE_BIT_CLR (hn->status, HE_STA_PUSHED);
            HE_BIT_CLR (hn->status, HE_STA_MOUSE_IN);
	    redraw = TRUE;
	    break; 

	case Expose :
	    redraw = TRUE;
	    break; 

    } /* switch */

    if (redraw) HeButtonDraw (hn);

    return 0;
}


/*
 * Dessin du bouton.
 *
*/

void HeButtonDraw (He_node *hn)
{
    OBJ_TYPE *ob = hn->OBJ_UNION;
    Window win = hn->win;
    int show     = HE_BIT_ISSET (hn->status, HE_STA_SHOW),
        pushed   = HE_BIT_ISSET (hn->status, HE_STA_PUSHED),
        mouse_in = HE_BIT_ISSET (hn->status, HE_STA_MOUSE_IN),
        active   = HE_BIT_ISSET (hn->status, HE_STA_ACTIVE),
        focus    = HE_BIT_ISSET (hn->status, HE_STA_FOCUS_IN),
        survol   = active && (mouse_in || pushed),
        c_bg     = (active && pushed && ob->xi == NULL) ? 
                       he_col_rev_bg : he_col_bg,
        c1_ext   = (pushed) ? he_col_dark_bd : he_col_light_bd,
        c2_ext   = (pushed) ? he_col_light_bd : he_col_dark_bd,
        c1_int   = (survol) ? he_col_dark_bd : c_bg,
        c2_int   = (survol) ? he_col_light_bd : c_bg,
        text     = (ob->label && ob->label[0]),
        y1       = 0;

/* printf ("draw button %s\n", ob->label); */

    /* Dessin du fond */
    HeDrawBg (hn, (show) ? c_bg : he_col_bg);
    if (!show) return;

    /* Dessin bord ou focus */
    if (focus && active)
        HeDrawRect3DC (win, hn->xb, hn->yb, hn->xm-1, hn->ym-1,
	    he_black, he_black );
    else
	HeDrawRect3DC (win, hn->xb, hn->yb, hn->xm-1, hn->ym-1,
	    c1_ext, c2_ext);

    /* Survole' par la souris */
    if (survol)
	HeDrawRect3D (win, hn->xb+1, hn->yb+1, hn->xm-3, hn->ym-3,
	    c1_int, c2_int);
    
    /* Calcul coordonne y1 */
    if (ob->xi && text) 
        y1 = hn->yb + (hn->ym - ob->xi->height - he_font_height) 
                      * HE_DECO_TW / (HE_DECO_TW + 2*HE_DECO_TH);
    
    /* Image */
    if (ob->xi) {
        int xb = hn->xb + (hn->xm - ob->xi->width) / 2,
            yb = (text) ? y1 : hn->yb + (hn->ym - ob->xi->height) / 2;
	HePutXi (hn->win, he_gc_draw, ob->xi, xb, yb);
    }
    
    /* Nom */
    if (text) {
        int yb = (ob->xi) ? y1 + ob->xi->height : hn->yb,
            ym = hn->ym - (yb - hn->yb);
    
        XSetForeground (he_display, he_gc_draw,
            (!active) ? he_col_light_bd :
            (pushed) ? he_col_rev_ink : he_col_ink );
        HeDrawStringCenterRect(win, he_gc_draw, he_normal_font,
            hn->xb, yb, hn->xm, ym, ob->label);
    }
}


/*
 * Mthode renvoyant &tip.
*/
He_tip_info *HeButtonStoreTip (He_node *hn)
{
    return &hn->OBJ_UNION->tip_info;
}


/*-------------------- F O N C T I O N S - P R I V E E S ---------------------*/

/*
 * Calcule la taille xm,ym du bouton selon label et xi
 *
*/
void HeButtonSize (He_node *hn, int deco_x, int deco_y, int *xm, int *ym)
{
    OBJ_TYPE *ob = hn->OBJ_UNION;
    int text = (ob->label && ob->label[0]);
    
    if (text) {
	HeDimString (he_normal_font, ob->label, xm, ym);
        if (ob->xi) {
            if (*xm < ob->xi->width) *xm = ob->xi->width;
	    *xm += deco_x;
            *ym += ob->xi->height + deco_y + deco_x/2;       
        } else {
	    *xm += deco_x;
            *ym += deco_y;
        }
    } else {
        if (ob->xi) {
	     *xm = ob->xi->width  + deco_x; 
             *ym = ob->xi->height + deco_x; /* bord large */
        } else {
	     *xm = deco_x; 
             *ym = deco_y + he_font_height; 
        }
    }
}


/*
 *  Teste nature du noeud ; renvoie TRUE si erreur. 
*/

int HeNotButton (He_node *hn)
{
    if (hn == NULL || hn->meth != &he_meth_button) {
	HeError ("Button expected\n");
        return TRUE;
    }
    return FALSE;
}

