/*
 * The Helium Toolkit
 *
 * Copyright (C) 1996-2000 Edouard Thiel <Edouard.Thiel@lif.univ-mrs.fr>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License; see http://www.lif-sud.univ-mrs.fr/~thiel/helium/LICENSE
*/

/*
 * gui/pan_message.c - 29/06/1999
 *
 * 
*/

#include <helium.h>

/* Type et union du package 
*/
#define  OBJ_TYPE	Obj_panel_message
#define  OBJ_UNION	obj.message

static He_method he_meth_message;


/*

  A FAIRE : prevoir que cet objet puisse etre aligne a droite ou a gauche ;
            -> modif automatique de xb qd xm change
            -> indiquer le bord droit absolu et memoriser.
  UTILE pour le footer.

*/


/*------------------- I N T E R F A C E - P U B L I Q U E --------------------*/

/*
 *  Cration d'un panel_message.
 *  Renvoie le noeud ou NULL.
*/

He_node *HeCreateMessage (He_node *owner)
{
    He_node *hn;
    OBJ_TYPE *ob;

    HeMessageAutoInit();

    /* On cree le noeud */
    hn = HeBaseCreate (owner, &he_meth_message);
    if (hn == NULL) return NULL;

    /* On cree l'objet */
    ob = hn->OBJ_UNION = malloc (sizeof (OBJ_TYPE));
    if (ob == NULL) {
	HeError ("HeCreateMessage: malloc error\n");
	return NULL;
    }

    /* Init des champs du noeud */    
    hn->xm = hn->ym = 0;		/* Inhibe le 1er HeUpdateDrawOldBg */
    hn->win = hn->owner->win;

    /* Init des champs de l'objet */
    ob->label = NULL;
    ob->bold = FALSE;
    ob->max_len = 80;
    HeTipInitInfo (&ob->tip_info);

    /* Positionne l'objet par rapport a PANEL_LAYOUT */
    HePosPanelItem (hn);

    /* Fixe taille par defaut, provoque HE_UPD_ANYDIM | HE_UPD_DRAW */
    HeMessageFit (hn);

    /* Fin succes */
    return hn;
}


/*
 * Creation d'un message parametre'
*/

He_node *HeCreateMessageP (He_node *owner, const char *label, int bold)
{
    He_node *tmp = HeCreateMessage (owner);
    HeSetMessageLabel (tmp, label);
    HeSetMessageBold (tmp, bold);
    return tmp;
}


/*
 * Set
*/

void HeSetMessageLabel (He_node *hn, const char *label)
{
    OBJ_TYPE *ob;
    
    if (HeNotMessage(hn)) return; ob = hn->OBJ_UNION;
    
    free (ob->label);
    if (label == NULL)
	ob->label = NULL;
    else if (ob->max_len <= 4 || strlen(label) <= ob->max_len)
	ob->label = strdup(label);
    else {
	char *tmp = strdup(label);
	if (tmp) tmp[ob->max_len-4] = 0;
	ob->label = malloc (ob->max_len + 1);
	if (ob->label && tmp) sprintf (ob->label, "%s ...", tmp);
	free (tmp);
    }
    if (HE_BIT_ISSET (hn->status, HE_STA_AUTOFIT)) HeMessageFit (hn);
}

void HeSetMessageBold (He_node *hn, int bold)
{
    if (HeNotMessage(hn)) return;
    hn->OBJ_UNION->bold = bold ? TRUE : FALSE;
    if (HE_BIT_ISSET (hn->status, HE_STA_AUTOFIT)) HeMessageFit (hn);
}


/* Si max_len <= 4, pas de limite de longueur
*/

void HeSetMessageMaxLen (He_node *hn, int max_len)
{
    OBJ_TYPE *ob;
    char *tmp;
    
    if (HeNotMessage(hn)) return; ob = hn->OBJ_UNION;
    
    ob->max_len = max_len;
    if (ob->label == NULL || max_len <= 4 || strlen(ob->label) <= max_len)
	return;
    
    tmp = ob->label;
    tmp[max_len-4] = 0;
    ob->label = malloc (max_len + 1);
    if (ob->label) sprintf (ob->label, "%s ...", tmp);
    free (tmp);
    HeMessageFit (hn);
}


/*
 * Get
*/

char *HeGetMessageLabel (He_node *hn)
{
    if (HeNotMessage(hn)) return he_empty_str;
    return hn->OBJ_UNION->label ? hn->OBJ_UNION->label : he_empty_str;
}

int HeGetMessageBold (He_node *hn)
{
    if (HeNotMessage(hn)) return FALSE;
    return hn->OBJ_UNION->bold;
}

int HeGetMessageMaxLen (He_node *hn)
{
    if (HeNotMessage(hn)) return 0;
    return hn->OBJ_UNION->max_len;
}


/*-------------------- M E T H O D E S - P R I V E E S ------------------------
 *
 * On ne teste pas si hn == NULL.
*/

/*
 * Initialisation du module ; appel par HeCreateMessage().
*/

void HeMessageAutoInit ()
{
    static int did_init = FALSE;
    
    if (did_init) return;
    
    HeMethodDefault (&he_meth_message, HE_NAT_MESSAGE);
    he_meth_message.destroy = HeMessageDestroy;
    he_meth_message.fit     = HeMessageFit;
    he_meth_message.update  = HeMessageUpdate;
    he_meth_message.event   = HeMessageEvent;
    he_meth_message.draw    = HeMessageDraw;
    he_meth_message.tip     = HeMessageStoreTip;
    
    did_init = TRUE;
}


/*
 * Destruction d'un panel_message ; appele' par HeDestroyRec().
*/

void HeMessageDestroy (He_node *hn)
{
    OBJ_TYPE *ob = hn->OBJ_UNION;

    free (ob->label);
    HeTipDestroyInfo (&ob->tip_info);

    free (ob);
}


/*
 * Ajuste la taille du panel_message en fonction de son contenu.
*/

void HeMessageFit (He_node *hn)
{
    OBJ_TYPE *ob;
    int x, y;
    
    if (HeNotMessage(hn)) return;
    ob = hn->OBJ_UNION;
    
    HeDimString ((ob->bold) ? he_bold_font : he_normal_font, ob->label, &x, &y);
    HeSetWidth (hn, x + HE_DECO_TW);
    HeSetHeight(hn, y + HE_DECO_TH);
    HeUpdateMark (hn, HE_UPD_DRAW);
}


/*
 * M-A-J geometrie et affichage ; appele' par HeUpdateFlush().
*/

void HeMessageUpdate (He_node *hn, He_upd_info *ui)
{
    OBJ_TYPE *ob = hn->OBJ_UNION;
    
    HeUpdateDrawOldBg (hn, ui);

    /* Se'curite' pour que le label ne de'passe pas ses frontieres */
    if (HE_BIT_ISSET (ui->mask, HE_UPD_SIZE)) {
	int x, y;
	HeDimString ((ob->bold) ? he_bold_font : he_normal_font,
	    ob->label, &x, &y);
	if (hn->xm < x) hn->xm = x;
	if (hn->ym < y + HE_DECO_TM) hn->ym = y + HE_DECO_TM;
    }

    if (HE_BIT_ISSET (ui->mask, HE_UPD_ANYDIM | HE_UPD_DRAW | HE_UPD_FOCUS)) 
	HeMessageDraw (hn);
}


/*
 * Methode pour gerer les evenements.
 * Renvoie -1 si erreur.
 *
 * Appele' par HePanelEvent()
*/

int HeMessageEvent (He_node *hn, He_event *hev)
{

    HeTipEvent (hn, hev);
    
    switch (hev->type) {

	case Expose :
	    HeMessageDraw (hn);
	    break; 

    } /* switch */

    return 0;
}


/*
 * Dessin du message.
 *
*/

void HeMessageDraw (He_node *hn)
{
    OBJ_TYPE *ob = hn->OBJ_UNION;
    Window win = hn->win;
    int show   = HE_BIT_ISSET (hn->status, HE_STA_SHOW),
        active = HE_BIT_ISSET (hn->status, HE_STA_ACTIVE);

    /* Dessin du fond */
    HeDrawBg (hn, he_col_bg);
    if (!show) return;

    /* Message */
    XSetForeground (he_display, he_gc_draw, 
        (active) ? he_col_ink : he_col_light_bd);
    HeDrawStringCenterRect (win, he_gc_draw, 
	(ob->bold) ? he_bold_font : he_normal_font,
	hn->xb, hn->yb, hn->xm, hn->ym, ob->label);
}


/*
 * Mthode renvoyant &tip.
*/
He_tip_info *HeMessageStoreTip (He_node *hn)
{
    return &hn->OBJ_UNION->tip_info;
}


/*-------------------- F O N C T I O N S - P R I V E E S ---------------------*/

/*
 *  Teste nature du noeud ; renvoie TRUE si erreur. 
*/

int HeNotMessage (He_node *hn)
{
    if (hn == NULL || hn->meth != &he_meth_message) {
	HeError ("Message expected\n");
        return TRUE;
    }
    return FALSE;
}


