/*
 * The Helium Toolkit
 *
 * Copyright (C) 1996-2000 Edouard Thiel <Edouard.Thiel@lif.univ-mrs.fr>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License; see http://www.lif-sud.univ-mrs.fr/~thiel/helium/LICENSE
*/

/*
 * gui/pan_text.h - 10/10/2000
 *
 * 
*/


#ifndef HELIUM__PAN_TEXT_H
#define HELIUM__PAN_TEXT_H


typedef struct Obj_panel_text_st {
	char	*value;			/* change avec realloc */
	Ulong	type;			/* int, float, text, passwd */
        int	str_len,		/* nb de char total */
		vis_len,		/* nd de char visibles */
		offset,			/* position text / fen */
		point;			/* point d'insertion / text */
	He_kbd_proc kbd_proc;		/* intercepte evt si retour FALSE */
	He_notify_proc notify_proc;	/* appelle' quand XK_Return */
        He_tip_info tip_info;
} Obj_panel_text;


/* PUBLIC */
extern He_node *HeCreateText (He_node *owner);
extern void HeSetTextValue (He_node *hn, const char *value);
extern void HeSetTextVisibleLen (He_node *hn, int len);
extern void HeSetTextPoint (He_node *hn, int point);
extern void HeSetTextOffset (He_node *hn, int offset);
extern void HeSetTextInsertMode (He_node *hn, int insert_mode);
extern void HeSetTextKbdProc (He_node *hn, He_kbd_proc proc);
extern void HeSetTextNotifyProc (He_node *hn, He_notify_proc proc);
extern void HeSetTextCompletion (He_node *hn, int completion);
extern char *HeGetTextValue (He_node *hn);
extern int HeGetTextLen (He_node *hn);
extern int HeGetTextVisibleLen (He_node *hn);
extern int HeGetTextPoint (He_node *hn);
extern int HeGetTextOffset (He_node *hn);
extern int HeGetTextInsertMode (He_node *hn);
extern He_kbd_proc HeGetTextKbdProc (He_node *hn);
extern He_notify_proc HeGetTextNotifyProc (He_node *hn);
extern int HeGetTextCompletion (He_node *hn);
extern int HeTextInsert (He_node *hn, const char *s, int len);
extern int HeTextOvertype (He_node *hn, const char *s, int len);
extern int HeTextDelete (He_node *hn, int len);
extern int HeTextInput (He_node *hn, char *s, int len, int delete_sel);
extern int HeTextSelIndex (He_node *hn, int sel1, int sel2);
extern int HeTextSelWord (He_node *hn);
extern int HeTextSelLine (He_node *hn);
extern int HeTextMoveSel (He_node *hn, int dec);
extern int HeTextPasteSel (He_node *hn);
extern int HeTextPasteClip (He_node *hn);
extern int HeTextCutClip (He_node *hn);
extern int HeTextCopyClip (He_node *hn);
extern int HeIsAlnum (int c);
extern int HeIsSpace (int c);
extern int HeIsPunct (int c);
extern int HeIsCntrl (int c);
extern void HeMoveTextPoint (He_node *hn, int move);
extern void HeMoveTextOffset (He_node *hn, int move);


/* PRIVE */
extern void HeTextAutoInit ();
extern void HeTextDestroy (He_node *hn);
extern void HeTextFit (He_node *hn);
extern void HeTextUpdate (He_node *hn, He_upd_info *ui);
extern int HeTextEvent (He_node *hn, He_event *hev);
extern void HeTextDraw (He_node *hn);
extern int HeTextSelExport (He_node *hn, Atom sel_type, Atom target,
	Atom *type, Uchar **value, int *length, int *format, int *free_value);
extern void HeTextSelImport (He_node *hn, Atom sel_type, Atom target, Atom prop,
	Atom type, int format, Ulong nitems, Uchar *data, int *free_data);
extern int HeTextCalcX0 (He_node *hn);
extern int HeTextCalcY0 (He_node *hn);
extern void HeTextDrawCursor (He_node *hn, int x0, int y0);
extern void HeTextDrawSelBg (He_node *hn, int x0, int y0);
extern void HeTextSel1Sel2 (int point);
extern void HeTextKeyPress (He_node *hn, He_event *hev);
extern int HeTextSymLen (char c);
extern int HeTextSymStrLen (const char *s, int len);
extern int HeTextSymMaxOffset (const char *s, int len, int vis);
extern int HeTextSymClic (He_node *hn, int x);
extern int HeTextSymPos (He_node *hn, int p);
extern int HeTextSymDepla (He_node *hn, int *dx, int p);
extern void HeTextSymDraw (He_node *hn, int x, int y);
extern He_tip_info *HeTextStoreTip (He_node *hn);
extern void HeTextBounds (He_node *hn);
extern int HeNotText (He_node *hn);


#endif /* HELIUM__PAN_TEXT_H */

