/*
 * The Helium Toolkit
 *
 * Copyright (C) 1996-2000 Edouard Thiel <Edouard.Thiel@lif.univ-mrs.fr>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License; see http://www.lif-sud.univ-mrs.fr/~thiel/helium/LICENSE
*/

/*
 * gui/pan_toggle.c - 21/02/2000
 *
 * Boutons  bascule : toggle, radio, check.
*/

#include <helium.h>

/* Type et union du package 
*/
#define  OBJ_TYPE	Obj_panel_toggle
#define  OBJ_UNION	obj.toggle

static He_method he_meth_toggle;


/* Dco : Width, Height, Left, Right, Bottom, Total-width
*/
#define HE_LED_W 12
#define HE_LED_H 7
#define HE_LED_L 6
#define HE_LED_B 5
#define HE_LED_T (HE_LED_W+HE_LED_L)

#define HE_CHK_W 12
#define HE_CHK_H 12
#define HE_CHK_L 4
#define HE_CHK_R 2
#define HE_CHK_T (HE_CHK_W+HE_CHK_L+HE_CHK_R)

#define HE_RAD_W 12
#define HE_RAD_H 12
#define HE_RAD_L 4
#define HE_RAD_R 2
#define HE_RAD_T (HE_RAD_W+HE_RAD_L+HE_RAD_R)


/*------------------- I N T E R F A C E - P U B L I Q U E --------------------*/

/*
 *  Cration d'un panel_toggle.
 *  Renvoie le noeud ou NULL.
*/

He_node *HeCreateToggle (He_node *owner, int categ)
{
    He_node *hn;
    OBJ_TYPE *ob;
    
    HeToggleAutoInit();
    
    /* On teste la categorie */
    if (categ != HE_LED && categ != HE_CHECK && categ != HE_RADIO) {
	HeError ("HeCreateToggle: bad category\n");
	return NULL;
    }

    /* On cree le noeud */
    hn = HeBaseCreate (owner, &he_meth_toggle);
    if (hn == NULL) return NULL;

    /* On cree l'objet */
    ob = hn->OBJ_UNION = malloc (sizeof (OBJ_TYPE));
    if (ob == NULL) {
	HeError ("HeCreateToggle: malloc error\n");
	return NULL;
    }

    /* Init des champs du noeud */    
    hn->xm = hn->ym = 0;		/* Inhibe le 1er HeUpdateDrawOldBg */
    hn->win = hn->owner->win;

    /* init des champs de l'objet */
    ob->label = NULL;
    ob->value = FALSE;
    ob->categ = categ;
    ob->next = hn;
    ob->notify_proc = NULL;
    HeTipInitInfo (&ob->tip_info);

    /* Positionne l'objet par rapport a PANEL_LAYOUT */
    HePosPanelItem (hn);

    /* Fixe taille par defaut, provoque HE_UPD_ANYDIM | HE_UPD_DRAW */
    HeToggleFit (hn);

    /* Fin succes */
    return hn;
}


/*
 * Creation d'un toggle parametre'
*/

He_node *HeCreateToggleLedP (He_node *owner, const char *label, 
		He_notify_proc proc, int value)
{
    He_node *tmp = HeCreateToggle (owner, HE_LED);
    HeSetToggleLabel (tmp, label);
    HeSetToggleNotifyProc (tmp, proc);
    HeSetToggleValue (tmp, value);
    return tmp;
}

He_node *HeCreateToggleCheckP (He_node *owner, const char *label, 
		He_notify_proc proc, int value)
{
    He_node *tmp = HeCreateToggle (owner, HE_CHECK);
    HeSetToggleLabel (tmp, label);
    HeSetToggleNotifyProc (tmp, proc);
    HeSetToggleValue (tmp, value);
    return tmp;
}

He_node *HeCreateToggleRadioP (He_node *owner, const char *label, 
		He_notify_proc proc, He_node *group)
{
    He_node *tmp = HeCreateToggle (owner, HE_RADIO);
    HeSetToggleLabel (tmp, label);
    HeSetToggleNotifyProc (tmp, proc);
    if (group == NULL)
         HeSetToggleValue (tmp, TRUE);
    else HeGroupToggle (tmp, group);
    return tmp;
}


/*
 * Set
*/

void HeSetToggleLabel (He_node *hn, const char *label)
{
    OBJ_TYPE *ob;
    
    if (HeNotToggle(hn)) return;
    ob = hn->OBJ_UNION;
    
    free (ob->label);
    ob->label = (label == NULL) ? NULL : strdup(label);
    
    if (HE_BIT_ISSET (hn->status, HE_STA_AUTOFIT)) HeToggleFit (hn);
}

void HeSetToggleValue (He_node *hn, int value)
{
    if (HeNotToggle(hn)) return;
    hn->OBJ_UNION->value = value ? TRUE : FALSE;
    HeUpdateMark (hn, HE_UPD_DRAW);
}

void HeSetToggleNotifyProc (He_node *hn, He_notify_proc proc)
{
    if (HeNotToggle(hn)) return;
    hn->OBJ_UNION->notify_proc = proc;
}


/*
 * Get
*/

char *HeGetToggleLabel (He_node *hn)
{
    if (HeNotToggle(hn)) return he_empty_str;
    return hn->OBJ_UNION->label ? hn->OBJ_UNION->label : he_empty_str;
}

int HeGetToggleValue (He_node *hn)
{
    if (HeNotToggle(hn)) return FALSE;
    return hn->OBJ_UNION->value;
}

He_notify_proc HeGetToggleNotifyProc (He_node *hn)
{
    if (HeNotToggle(hn)) return FALSE;
    return hn->OBJ_UNION->notify_proc;
}


/*
 * ------- A utiliser sur les boutons Radio : -------
 * --- (mais marche aussi pour les autres toggle) ---
*/

/* Groupe des boutons, en fusionnant les listes circulaires.
 * N'a aucun effet sur value. 
*/

void HeGroupToggle (He_node *h1, He_node *h2)
{
    OBJ_TYPE *o1, *o2;
    He_node *tmp;
    
    if (HeNotToggle(h1) || HeNotToggle(h2)) return;
    o1 = h1->OBJ_UNION; o2 = h2->OBJ_UNION;
    
    /* Test si les listes sont bien disjointes */
    tmp = h1;
    do {
        if (tmp == h2) return;
        tmp = tmp->OBJ_UNION->next;
    } while (tmp && tmp != h1);
    
    /* Marche uniquement si les listes sont disjointes */
    tmp = o2->next; o2->next = o1->next; o1->next = tmp;
}


/* Isole un bouton, en le dtachant de sa liste circulaire.
 * N'a aucun effet sur value. 
*/

void HeUngroupToggle (He_node *hn)
{
    He_node *tmp, *ha;
    OBJ_TYPE *ob, *oa;
    
    if (HeNotToggle(hn)) return;
    
    /* Recherche de l'antcdent ha de hn */
    tmp = hn;
    do {
        ha = tmp; tmp = tmp->OBJ_UNION->next;
    } while (tmp && tmp != hn);
    
    /* Sparation (marche dans tous les cas) */
    ob = hn->OBJ_UNION; oa = ha->OBJ_UNION;
    oa->next = ob->next; ob->next = hn;
}


/* Renvoie le bouton du groupe qui est allum (le premier trouv),
 * sinon NULL.
*/

He_node *HeGetToggleLighted (He_node *hn)
{
    He_node *ha;
    OBJ_TYPE *oa;
    
    if (HeNotToggle(hn)) return NULL;

    ha = hn;
    do {
        oa = ha->OBJ_UNION;
	if (oa->value) return ha;
        ha = oa->next;
    } while (ha && ha != hn);
    
    return NULL;
}


/*
 * Change le bouton du groupe qui est allum :
 *  - allume hn et teint l'ancien bouton allum ha (si ha != hn).
 *  - appelle les notify_proc de hn et de ha.
 * Si ce n'est pas un groupe, fait on/off + notify_proc.
*/

void HeSetToggleLighted (He_node *hn)
{
    He_node *ha;
    OBJ_TYPE *ob, *oa;
    
    if (HeNotToggle(hn)) return;
    ob = hn->OBJ_UNION;

    /* C'est un bouton non group
    */
    if (ob->next == NULL || ob->next == hn) {
	ob->value = !ob->value;
	HeUpdateMark (hn, HE_UPD_DRAW);
	if (ob->notify_proc != NULL)
	    (ob->notify_proc)(hn);
	return;
    } 
     
    /* C'est un bouton group : on rcupre l'ancien bouton allum ha.
    */
    ha = HeGetToggleLighted (hn);
    
    /* C'est le bouton dj allum qui est press */
    if (ha == hn) return;
    
    /* On teint ha */
    if (ha != NULL) {
        oa = ha->OBJ_UNION;
        oa->value = FALSE;
	HeUpdateMark (ha, HE_UPD_DRAW);
	if (oa->notify_proc != NULL)
            (oa->notify_proc)(ha);
    } 
    
    /* On allume hn */
    ob->value = TRUE;
    HeUpdateMark (hn, HE_UPD_DRAW);
    if (ob->notify_proc != NULL)
        (ob->notify_proc)(hn);    
}


/*-------------------- M E T H O D E S - P R I V E E S ------------------------
 *
 * On ne teste pas si hn == NULL.
*/

/*
 * Initialisation du module ; appel par HeCreateToggle().
*/

void HeToggleAutoInit ()
{
    static int did_init = FALSE;
    
    if (did_init) return;
    
    HeMethodDefault (&he_meth_toggle, HE_NAT_TOGGLE);
    he_meth_toggle.props   = HE_PM_ACCEPT_FOCUS;
    he_meth_toggle.destroy = HeToggleDestroy;
    he_meth_toggle.fit     = HeToggleFit;
    he_meth_toggle.update  = HeToggleUpdate;
    he_meth_toggle.event   = HeToggleEvent;
    he_meth_toggle.draw    = HeToggleDraw;
    he_meth_toggle.tip     = HeToggleStoreTip;
    
    did_init = TRUE;
}


/*
 * Destruction d'un panel_toggle ; appele' par HeDestroyRec().
*/

void HeToggleDestroy (He_node *hn)
{
    OBJ_TYPE *ob = hn->OBJ_UNION;

    HeUngroupToggle (hn);
    free (ob->label);
    HeTipDestroyInfo (&ob->tip_info);
    free (ob);
}


/*
 * Ajuste la taille du panel_toggle en fonction de son contenu.
*/

void HeToggleFit (He_node *hn)
{
    int xm, ym;
    
    if (HeNotToggle(hn)) return;
    
    HeToggleSize (hn, HE_DECO_TW, HE_DECO_TH, &xm, &ym);
    HeSetWidth (hn, xm);
    HeSetHeight(hn, ym);
    HeUpdateMark (hn, HE_UPD_DRAW);
}


/*
 * M-A-J geometrie et affichage ; appele' par HeUpdateFlush().
*/

void HeToggleUpdate (He_node *hn, He_upd_info *ui)
{
    HeUpdateDrawOldBg (hn, ui);

    /* Se'curite' pour que le nom ne de'passe pas le bouton */
    if (HE_BIT_ISSET (ui->mask, HE_UPD_SIZE)) {
	int x, y;

	HeToggleSize (hn, HE_DECO_TM, HE_DECO_TM, &x, &y);
	if (hn->xm < x) hn->xm = x;
	if (hn->ym < y) hn->ym = y;
    }

    if (HE_BIT_ISSET (ui->mask, HE_UPD_ANYDIM | HE_UPD_DRAW | HE_UPD_FOCUS)) 
	HeToggleDraw (hn);
}


/*
 * Methode pour gerer les evenements.
 * Renvoie -1 si erreur.
 *
 * Appele' par HePanelEvent()
*/

int HeToggleEvent (He_node *hn, He_event *hev)
{
    int redraw = FALSE;

#ifdef DEBUG
    OBJ_TYPE *ob = hn->OBJ_UNION;
    printf ("    toggle %s %s\n", ob->label, HeGetXEventName (hev->type)); 
#endif

    HeTipEvent (hn, hev);
    
    switch (hev->type) {

	case KeyPress :
            if (!HE_BIT_ISSET (hn->status, HE_STA_ACTIVE)) break;
            if (!(hev->sym == XK_Return ||
                  hev->sym == XK_KP_Enter ||
                  hev->sym == XK_ISO_Enter)) break;
            if (HE_BIT_ISSET (hn->status, HE_STA_PUSHED)) break;
	    HE_BIT_SET (hn->status, HE_STA_PUSHED);
	    redraw = TRUE;
	    break; 
            
	case ButtonPress :
            if (!HE_BIT_ISSET (hn->status, HE_STA_ACTIVE)) break;
            if (HE_BIT_ISSET (hn->status, HE_STA_PUSHED)) break;
            if (HE_BIT_ISSET (hev->xev->xbutton.state, ShiftMask))
		 HeSetFocus (hn, TRUE); 
	    else HE_BIT_SET (hn->status, HE_STA_PUSHED);
	    redraw = TRUE;
	    break; 

	case KeyRelease :
            if (!(hev->sym == XK_Return ||
                  hev->sym == XK_KP_Enter ||
                  hev->sym == XK_ISO_Enter)) break;
	case ButtonRelease :
            if (!HE_BIT_ISSET (hn->status, HE_STA_ACTIVE)) break;
	    if (!HE_BIT_ISSET (hn->status, HE_STA_PUSHED)) break;
	    HE_BIT_CLR (hn->status, HE_STA_PUSHED);
            /* On ne met pas redraw=TRUE car HeSetToggleLighted met
               dans certains cas le flag HE_UPD_DRAW */
	    HeUpdateMark (hn, HE_UPD_DRAW); /* systmatique */
            HeSetToggleLighted (hn);
	    break; 

	case MotionNotify :
	    break;	

	case EnterNotify : 
	    HE_BIT_SET (hn->status, HE_STA_MOUSE_IN);
	    redraw = TRUE;
	    break; 

	case LeaveNotify : 
	    HE_BIT_CLR (hn->status, HE_STA_PUSHED);
	    HE_BIT_CLR (hn->status, HE_STA_MOUSE_IN);
	    redraw = TRUE;
	    break; 

	case Expose :
	    redraw = TRUE;
	    break; 

    } /* switch */

    if (redraw) HeToggleDraw (hn);

    return 0;
}


/*
 * Dessin du bouton.
 *
*/

void HeToggleDraw (He_node *hn)
{
    OBJ_TYPE *ob = hn->OBJ_UNION;
    Window win = hn->win;
    int show     = HE_BIT_ISSET (hn->status, HE_STA_SHOW),
        pushed   = HE_BIT_ISSET (hn->status, HE_STA_PUSHED),
        mouse_in = HE_BIT_ISSET (hn->status, HE_STA_MOUSE_IN),
        active   = HE_BIT_ISSET (hn->status, HE_STA_ACTIVE),
        focus    = HE_BIT_ISSET (hn->status, HE_STA_FOCUS_IN),
        survol   = active && (mouse_in || pushed);
        
    switch (ob->categ) {
    
        case HE_LED : {
        
            int c_bg   = (active && pushed) ? he_col_rev_bg : he_col_bg,
                c1_ext = (pushed) ? he_col_dark_bd : he_col_light_bd,
                c2_ext = (pushed) ? he_col_light_bd : he_col_dark_bd,
                c1_int = (survol) ? he_col_dark_bd : c_bg,
                c2_int = (survol) ? he_col_light_bd : c_bg;

            /* Dessin du fond */
            HeDrawBg (hn, (show) ? c_bg : he_col_bg);
            if (!show) return;

            /* Dessin bord focus */
            if (focus)
                HeDrawRect3DC (win, hn->xb, hn->yb, hn->xm-1, hn->ym-1,
        	    he_black, he_black );
            else
        	HeDrawRect3DC (win, hn->xb, hn->yb, hn->xm-1, hn->ym-1,
                    c1_ext, c2_ext);

            /* Survole' par la souris */
            if (survol)
                HeDrawRect3D (win, hn->xb+1, hn->yb+1, hn->xm-3, hn->ym-3,
                   c1_int, c2_int);
 
 	    /* Led */
	    HeDrawToggleLed (win,
                hn->xb+HE_LED_L, 
                hn->yb+hn->ym-HE_LED_B-HE_LED_H, 
                c2_ext, c1_ext,
                ob->value && active ? he_col_toggle : c_bg);
                        
            /* Nom */
            XSetForeground (he_display, he_gc_draw,
                (!active) ? he_col_light_bd :
                (pushed) ? he_col_rev_ink : he_col_ink );
            HeDrawStringCenterRect(win, he_gc_draw, he_normal_font,
                hn->xb+HE_LED_T, hn->yb, 
                hn->xm-HE_LED_T, hn->ym, ob->label);
        } break;
            
        case HE_CHECK : {
        
            int allume = active && ob->value && !pushed,
                c_bg   = (active && pushed) ? he_col_rev_bg : he_col_bg,
                c1_ext = (survol) ? he_col_dark_bd : he_col_light_bd,
                c2_ext = (survol) ? he_col_light_bd : he_col_dark_bd;

            /* Dessin du fond */
            HeDrawBg (hn, he_col_bg);
            if (!show) return;

            /* Dessin bord focus */
            if (focus)
                HeDrawRect3DC (win, hn->xb, hn->yb, hn->xm-1, hn->ym-1,
        	    he_black, he_black );
  
 	    /* Check */
	    HeDrawToggleCheck (win,
                hn->xb+HE_CHK_L, 
                hn->yb+(hn->ym-HE_CHK_H)/2, 
                c1_ext, c2_ext, 
                allume ? he_col_toggle : c_bg);
                        
            /* Nom */
            XSetForeground (he_display, he_gc_draw,
                (!active) ? he_col_light_bd : he_col_ink );
            HeDrawStringCenterRect(win, he_gc_draw, he_normal_font,
                hn->xb+HE_CHK_T, hn->yb, 
                hn->xm-HE_CHK_T, hn->ym, ob->label);
        } break;
            
        case HE_RADIO : {
        
            int allume = active && ob->value && !pushed,
                c_bg   = (active && pushed) ? he_col_rev_bg : he_col_bg,
                c1_ext = (survol) ? he_col_dark_bd : he_col_light_bd,
                c2_ext = (survol) ? he_col_light_bd : he_col_dark_bd;
 
            /* Dessin du fond */
            HeDrawBg (hn, he_col_bg);
            if (!show) return;

            /* Dessin bord focus */
            if (focus)
                HeDrawRect3DC (win, hn->xb, hn->yb, hn->xm-1, hn->ym-1,
        	    he_black, he_black );
  
 	    /* Radio */
	    HeDrawToggleRadio (win,
                hn->xb+HE_RAD_L, 
                hn->yb+(hn->ym-HE_RAD_H)/2, 
                c1_ext, c2_ext,
                allume ? he_col_toggle : c_bg);
                        
            /* Nom */
            XSetForeground (he_display, he_gc_draw,
                (!active) ? he_col_light_bd : he_col_ink );
            HeDrawStringCenterRect(win, he_gc_draw, he_normal_font,
                hn->xb+HE_RAD_T, hn->yb, 
                hn->xm-HE_RAD_T, hn->ym, ob->label);
       } break;
    }
}


/*
 * Mthode renvoyant &tip.
*/
He_tip_info *HeToggleStoreTip (He_node *hn)
{
    return &hn->OBJ_UNION->tip_info;
}


/*-------------------- F O N C T I O N S - P R I V E E S ---------------------*/

/*
 * Dessine un Led
*/
void HeDrawToggleLed (Window win, int x, int y, int c1, int c2, int cbg)
{
    HeDrawRect3D (win, x, y, HE_LED_W-1, HE_LED_H-1, c1, c2);
    XSetForeground (he_display, he_gc_draw, cbg);
    XFillRectangle (he_display, win, he_gc_draw,
        x+1, y+1, HE_LED_W-2, HE_LED_H-2);        
}


/*
 * Dessine un Check
*/
void HeDrawToggleCheck (Window win, int x, int y,
    int c1, int c2, int cbg)
{
    HeDrawRect3D (win, x, y, HE_CHK_W-1, HE_CHK_H-1, c1, c2);
    XSetForeground (he_display, he_gc_draw, cbg);
    XFillRectangle (he_display, win, he_gc_draw,
            x+2, y+2, HE_CHK_W-4, HE_CHK_H-4);        
}


/*
 * Dessine un Radio
*/
void HeDrawToggleRadio (Window win, int x, int y,
    int c1, int c2, int cbg)
{
    XSetForeground (he_display, he_gc_draw, c1);
    XDrawArc (he_display, win, he_gc_draw,
    	x, y, HE_RAD_W-1, HE_RAD_H-1, 40*64, 185*64);
    XSetForeground (he_display, he_gc_draw, c2);
    XDrawArc (he_display, win, he_gc_draw,
    	x, y, HE_RAD_W-1, HE_RAD_H-1, 225*64, 175*64);
        
    XSetForeground (he_display, he_gc_draw, cbg);
    XFillRectangle (he_display, win, he_gc_draw,
	x+HE_RAD_W/2-2, y+2, 4, HE_RAD_H-4);
    XFillRectangle (he_display, win, he_gc_draw,
	x+2, y+HE_RAD_H/2-2, HE_RAD_W-4, 4);
}


/*
 * Calcule la taille xm,ym du bouton selon label et categ
 *
*/
void HeToggleSize (He_node *hn, int deco_x, int deco_y, int *xm, int *ym)
{
    OBJ_TYPE *ob = hn->OBJ_UNION;
    
    switch (ob->categ) {
        case HE_LED :
            if (ob->label && ob->label[0]) {
                HeDimString (he_normal_font, ob->label, xm, ym);
                *xm += deco_x + HE_LED_T;
                *ym += deco_y;
            } else {
                *xm = HE_LED_T + HE_LED_L;
                *ym = deco_y + he_font_height;
            }
            break;
        case HE_CHECK :
            if (ob->label && ob->label[0]) {
                HeDimString (he_normal_font, ob->label, xm, ym);
                *xm += deco_x + HE_CHK_T;
                *ym += deco_y;
            } else {
                *xm = HE_CHK_W+HE_CHK_L*2;
                *ym = deco_y + he_font_height;
            }
            break;
        case HE_RADIO :
            if (ob->label && ob->label[0]) {
                HeDimString (he_normal_font, ob->label, xm, ym);
                *xm += deco_x + HE_RAD_T;
                *ym += deco_y;
            } else {
                *xm = HE_RAD_W+HE_RAD_L*2;
                *ym = deco_y + he_font_height;
            }
            break;
    }
}


/*
 *  Teste nature du noeud ; renvoie TRUE si erreur. 
*/

int HeNotToggle (He_node *hn)
{
    if (hn == NULL || hn->meth != &he_meth_toggle) {
	HeError ("Toggle expected\n");
        return TRUE;
    }
    return FALSE;
}

