/*
 * The Helium Toolkit
 *
 * Copyright (C) 1996-2000 Edouard Thiel <Edouard.Thiel@lif.univ-mrs.fr>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License; see http://www.lif-sud.univ-mrs.fr/~thiel/helium/LICENSE
*/

/*
 * gui/plug.c - 29/03/2000
 *
 * 
*/

#include <helium.h>

/* Type et union du package 
*/
#define  OBJ_TYPE	Obj_plug
#define  OBJ_UNION	obj.plug

static He_method he_meth_plug;


/*------------------- I N T E R F A C E - P U B L I Q U E --------------------*/

/*
 *  Cration d'un plug.
 *  Renvoie le noeud ou NULL.
*/

He_node *HeCreatePlug (He_node *owner)
{
    He_node *hn;
    OBJ_TYPE *ob;
    
    HePlugAutoInit();
    
    /* On cree le noeud */
    hn = HeBaseCreate (owner, &he_meth_plug);
    if (hn == NULL) return NULL;

    /* On cree l'objet */
    ob = hn->OBJ_UNION = malloc (sizeof (OBJ_TYPE));
    if (ob == NULL) {
	HeError ("HePlugCreate: malloc error\n");
	return NULL;
    }
    
    /* Init des champs du noeud */
    HE_BIT_CLR (hn->status, HE_STA_SHOW);
    hn->xm = hn->owner->xm;
    hn->ym = hn->owner->ym;

    /* Init des champs de l'objet */
    ob->old_xm = -1;    /* pour detecter les resize ; ici on */
    ob->old_ym = -1;    /* force un resize  la creation.    */
    
    /* Pas encore de Window */
    hn->win = 0;

    /* Fin succes */
    return hn;
}


/*
 * Set
*/

void HeSetPlugWindow (He_node *hn, Window win)
{
    if (HeNotPlug(hn) || hn->win == win) return;
    
    if (hn->win != 0) {
	XReparentWindow (he_display, hn->win, he_root_window, 0, 0);
	XMapRaised (he_display, hn->win);
	HeWinListRemove (hn);
    }
    
    hn->win = win;

    if (hn->win != 0) {
	/*XUnmapWindow (he_display, hn->win);*/
	XReparentWindow (he_display, hn->win, hn->owner->win, hn->xb, hn->yb);
	HeWinListInsert (hn);
	/*HeUpdateMark (hn, HE_UPD_DRAW);*/
    }
}


/*
 * Get
*/

Window HeGetPlugWindow (He_node *hn)
{
    if (HeNotPlug(hn)) return 0;
    return hn->win;
}
 

/*-------------------- M E T H O D E S - P R I V E E S ------------------------
 *
 * On ne teste pas si hn == NULL.
*/

/*
 * Initialisation du module ; appel par HeCreatePlug().
*/

void HePlugAutoInit ()
{
    static int did_init = FALSE;
    
    if (did_init) return;
    
    HeMethodDefault (&he_meth_plug, HE_NAT_PLUG);
    he_meth_plug.props   = HE_PM_WIN_OWNER;
    he_meth_plug.destroy = HePlugDestroy;
    he_meth_plug.fit     = HePlugFit;
    he_meth_plug.update  = HePlugUpdate;
    he_meth_plug.event   = HePlugEvent;
    
    did_init = TRUE;
}


/*
 * Destruction d'un plug ; appele' par HeDestroyRec().
*/

void HePlugDestroy (He_node *hn)
{
    OBJ_TYPE *ob = hn->OBJ_UNION;
    
    if (hn->win != 0) {
	XReparentWindow (he_display, hn->win, he_root_window, 0, 0);
	XMapRaised (he_display, hn->win);
	HeWinListRemove (hn);
    }

    free (ob);
}


/*
 * Ajuste la taille du plug en fonction de son contenu.
*/

void HePlugFit (He_node *hn)
{
    if (HeNotPlug(hn)) return;
    
    /* ??? */
}


/*
 * M-A-J geometrie et affichage ; appele' par HeUpdateFlush().
 *
*/

void HePlugUpdate (He_node *hn, He_upd_info *ui)
{
    if (hn->win == 0) return;
    
    HeUpdateWinGeom (hn, ui);

    if (HE_BIT_ISSET (ui->mask, HE_UPD_BORDER)) {
	XSetWindowBorderWidth (he_display, hn->win, hn->win_border);
    }

    if (HE_BIT_ISSET (ui->mask, HE_UPD_DRAW)) {
	if (HE_BIT_ISSET (hn->status, HE_STA_SHOW)) 
	     XMapWindow   (he_display, hn->win);
	else XUnmapWindow (he_display, hn->win);
    }
}


/*
 * Gestion evenements X ; appele' par HeEventDispatch().
 * Renvoie -1 si erreur.
*/

int HePlugEvent (He_node *hn, He_event *hev)
{

    switch (hev->type) {

    } /* switch */

    return 0;
}


/*-------------------- F O N C T I O N S - P R I V E E S ---------------------*/

/*
 *  Teste nature du noeud ; renvoie TRUE si erreur. 
*/

int HeNotPlug (He_node *hn)
{
    if (hn == NULL || hn->meth != &he_meth_plug) {
	HeError ("Plug expected\n");
        return TRUE;
    }
    return FALSE;
}

