/*
 * The Helium Toolkit
 *
 * Copyright (C) 1996-2000 Edouard Thiel <Edouard.Thiel@lif.univ-mrs.fr>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License; see http://www.lif-sud.univ-mrs.fr/~thiel/helium/LICENSE
*/

/*
 * gui/winlist.c - 28/03/2000
 *
 * Liste de noeuds tris dans l'ordre croissant des Window.
 * But : retrouver trs vite un noeud  partir d'un Window dans
 *       HeEventDispatch().
*/

#include <helium.h>

static He_vlist he_win_list;



/*------------------- I N T E R F A C E - P U B L I Q U E --------------------*/


/*
 * Recherche rapide d'un He_node  partir de son win.
 * Renvoie *noeud ou NULL.
 * Silencieux.
*/

He_node *HeGetNodeFromWin (Window win)
{
    He_node st_elt, *elt = &st_elt;
    int index, res;
    
    elt->win = win;
    res = HeVListSearch (&he_win_list, &elt, &index);
    return (res == 0) ? ((He_node**)he_win_list.v)[index] : NULL;
}


/*-------------------- F O N C T I O N S - P R I V E E S ---------------------*/

/*
 * Initialisation generale du module.
*/

void HeWinListInit ()
{
    /* Liste de noeuds trie dans l'ordre croissant des win
    */
    HeVListInit (&he_win_list, TRUE, 32, sizeof(He_node*), HeWinListComparWin);
}


/*
 * Callback de comparaison pour HeVListSearch()
*/

int HeWinListComparWin (void *v1, void *v2)
{
    return (*(He_node **)v1)->win == (*(He_node **)v2)->win ?  0 :
           (*(He_node **)v1)->win <  (*(He_node **)v2)->win ? -1 : 1;
}


/*
 * Attachement dans he_win_list (agrandissement automatique).
 * Si hn est dj dans la liste, ne fait rien.
 * Renvoie 0 succs, -1 erreur.
*/

int HeWinListInsert (He_node *hn)
{
    if (hn == NULL) return -1;

    return (HeVListInsert (&he_win_list, &hn) < 0) ? -1 : 0;
}


/*
 * Dtachement de he_win_list (rduction automatique).
 * Renvoie 0 succs, -1 erreur.
*/

int HeWinListRemove (He_node *hn)
{
    int index;
    
    if (hn == NULL) return -1;

    /* Recherche de la position dans la liste */
    if (HeVListSearch (&he_win_list, &hn, &index) < 0) return -1;

    /* Suppression de la liste */
    if (HeVListRemove (&he_win_list, index) < 0) return -1;
    
    return 0;
}


/*
 * Affiche infos pour debugage
*/

void HeWinListDebug ()
{
    int i;

    printf ("node_nb = %d   node_max = %d\n", 
        he_win_list.nb, he_win_list.max);

    for (i = 0; i < he_win_list.nb; i++)
	printf ("  [%d]  win = 0x%x\n", 
            i, (int)((He_node**)he_win_list.v)[i]->win);
}


