/*
 * The Helium Toolkit
 *
 * Copyright (C) 1996-2000 Edouard Thiel <Edouard.Thiel@lif.univ-mrs.fr>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License; see http://www.lif-sud.univ-mrs.fr/~thiel/helium/LICENSE
*/

/*
 * include/macros.h - 02/03/2001
 *
 * Declaration des macros
*/


#ifndef HELIUM__MACROS_H
#define HELIUM__MACROS_H


/*
 * Macros pour manipuler des ensembles de bits avec des masques de bits.
 * En gnral, ensemble et masque sont cods sur un unsigned long :
 *
 *  - HE_BIT_ZERO   vide l'ensemble ;
 *  - HE_BIT_SET    allume les bits du masque dans l'ensemble ;
 *  - HE_BIT_CLR    teint les bits du masque de l'ensemble ;
 *  - HE_BIT_ISSET  renvoie TRUE si l'un au moins des bits du masque est 
 *                  dans l'ensemble, renvoie FALSE sinon.
*/

#define HE_BIT_ZERO(set)  ((set)=0)

#define HE_BIT_SET(set,mask)  ((set) |= (mask))

#define HE_BIT_CLR(set,mask)  ((set) &= ~(mask))

#define HE_BIT_ISSET(set,mask)  (((set) & (mask)) ? TRUE : FALSE)



#endif /* HELIUM__MACROS_H */

